@@ -50,7 +50,7 @@ PyObject *RPS_read_video(int channel);
 void RPS_sample_surfaces(PyObject *rgb, PyObject *rgba);
 void RPS_set_video(int channel, int video);
 
-void RPS_init(int freq, int stereo, int samples, int status, int equal_mono);
+void RPS_init(int freq, int stereo, int samples, int status, int equal_mono, int linear_fades);
 void RPS_quit(void);
 
 void RPS_advance_time(void);
@@ -58,4 +58,7 @@ void RPS_periodic(void);
 
 char *RPS_get_error(void);
 
+extern void (*RPS_generate_audio_c_function)(float *stream, int length);
+
+
 #endif
