@@ -1,4 +1,4 @@
-# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2023 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -410,8 +410,6 @@ class TransitionAnimation(renpy.display.core.Displayable):
 
 
 class Blink(renpy.display.core.Displayable):
-    """
-    """
 
     def __init__(self, image, on=0.5, off=0.5, rise=0.5, set=0.5, # @ReservedAssignment
                  high=1.0, low=0.0, offset=0.0, anim_timebase=False, **properties):
@@ -560,7 +558,8 @@ def Filmstrip(image, framesize, gridsize, delay, frames=None, loop=True, **prope
             x = c * width
             y = r * height
 
-            args.append(renpy.display.im.Crop(image, x, y, width, height))
+            args.append(renpy.display.transform.Transform(
+                image, crop=(x, y, width, height)))
             args.append(delay)
 
             i += 1
