@@ -1,4 +1,4 @@
-# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2023 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -171,19 +171,87 @@ class Drag(renpy.display.core.Displayable, renpy.revertable.RevertableObject):
         to the corner of this drag.
 
     `drag_offscreen`
-        If true, this draggable can be moved offscreen. This can be
+        Determines the conditions under which the drag is allowed
+        to be dragged offscreen. Allowing offscreen dragging can be
         dangerous to use with drag_joined or drags that can change
         size, as the drags can leave the screen entirely, with no
         way to get them back on the screen.
 
+        This should be one of:
+
+        False
+            To disallow dragging the drag offscreen. (The default)
+
+        True
+            To allow dragging offscreen, in any direction.
+
+        "horizontal"
+            To allow dragging offscreen in the horizontal direction only.
+
+        "vertical"
+            To allow dragging offscreen in the vertical direction only.
+
+        (width, height)
+            Both width and height must be integers. The drag can be
+            dragged offscreen as long as a (width, height)-sized part
+            of it remains on-screen. So, (100, 100) will ensure that
+            at least a 100x100 pixel area of the displayable will
+            remain on-screen even while the rest of the displayable
+            can be dragged offscreen. Setting this to the width and
+            height of the displayable being dragged is equivalent to
+            not allowing the drag to go offscreen at all.
+
+        (min_x, max_x, min_y, max_y)
+            Where each of min_x, max_x, min_y, and max_y are integers.
+            min_x is the number of pixels away from the left border,
+            and max_x is the number of pixels away from the right
+            border. The same goes for min_y and max_y on the top and
+            bottom borders respectively. The drag can be moved until
+            one of its edges hit the specified border. (0, 0, 0, 0)
+            is equivalent to not allowing dragging offscreen at all.
+
+            For example, (-100, 200, 0, 0) would allow the drag to be
+            dragged 100 pixels off the left edge of the screen and 200
+            pixels off the right edge of the screen, but does not
+            allow it to be dragged offscreen at the top nor bottom.
+
+            This can also be used to constrain the drag within the
+            screen bounds. (200, -200, 200, -200) would only allow
+            the drag within 200 pixels of the edges of the screen.
+
+            You can envision this as an additional "border" around
+            the drag, which may go outside the bounds of the screen,
+            that constrains the drag to remain within it.
+
+        callable
+            A callable can be provided to drag_offscreen. It must
+            take two arguments: an x and a y position which
+            represents the dragged position of the top left corner of
+            the drag, and it must return an (x, y) tuple which is the
+            new (x, y) position the drag should be in. This callable
+            is called frequently, whenever the drag is moved. For
+            example, the following function snaps the drag into place
+            every 300 pixels::
+
+                def drag_snap(x, y):
+
+                    if y < 300:
+                        y = 0
+                    elif y < 600:
+                        y = 300
+                    else:
+                        y = 600
+
+                    return 200, y
+
     `mouse_drop`
         If true, the drag is dropped on the first droppable under the cursor.
         If false, the default, the drag is dropped onto the droppable with
         the largest degree of overlap.
 
     `drop_allowable`
-        A callback that is called to determine whether this drop allow
-        the current drags dropped onto. It is called with two arguments.
+        A callback that is called to determine whether this drop will allow
+        the current drags to be dropped onto it. It is called with two arguments.
         The first is the Drag which determines its sensitivity.
         The second is a list of Drags that are being dragged.
 
@@ -344,6 +412,10 @@ class Drag(renpy.display.core.Displayable, renpy.revertable.RevertableObject):
         if d is not None:
             self.add(d)
 
+    @property
+    def _draggable(self):
+        return self.draggable
+
     def snap(self, x, y, delay=0):
         """
         :doc: drag_drop method
@@ -387,6 +459,7 @@ class Drag(renpy.display.core.Displayable, renpy.revertable.RevertableObject):
             raise Exception("Drag expects either zero or one children.")
 
         self.child = renpy.easy.displayable(d)
+        renpy.display.render.invalidate(self)
 
     def _clear(self):
         self.child = None
@@ -399,8 +472,8 @@ class Drag(renpy.display.core.Displayable, renpy.revertable.RevertableObject):
         Changes the child of this drag to `d`.
         """
 
-        d.per_interact()
         self.child = renpy.easy.displayable(d)
+        self.child.per_interact()
         renpy.display.render.invalidate(self)
 
     def top(self):
@@ -525,7 +598,7 @@ class Drag(renpy.display.core.Displayable, renpy.revertable.RevertableObject):
             self.target_at = at + self.target_at_delay
             self.target_at_delay = 0
             redraw(self, 0)
-        elif at >= self.target_at:
+        elif self.target_at <= at or self.target_at <= self.at:
             # Snap complete
             self.x = self.target_x
             self.y = self.target_y
@@ -687,12 +760,37 @@ class Drag(renpy.display.core.Displayable, renpy.revertable.RevertableObject):
                     new_x = int(par_x - self.grab_x + xo) # type: ignore
                     new_y = int(par_y - self.grab_y + yo) # type: ignore
 
-                    if not self.drag_offscreen:
+                    # Constrain x-axis
+                    if not self.drag_offscreen or self.drag_offscreen == "vertical":
                         new_x = max(new_x, 0)
                         new_x = min(new_x, int(i.parent_width - i.w))
+                    # Constrain y-axis
+                    if not self.drag_offscreen or self.drag_offscreen == "horizontal":
                         new_y = max(new_y, 0)
                         new_y = min(new_y, int(i.parent_height - i.h))
 
+                    if isinstance(self.drag_offscreen, tuple):
+                        if len(self.drag_offscreen) not in (2, 4):
+                            raise Exception("Invalid number of arguments to drag_offscreen.")
+
+                        # Tuple of (x_min, x_max, y_min, y_max)
+                        if len(self.drag_offscreen) == 4:
+                            x_min, x_max, y_min, y_max = self.drag_offscreen
+                            new_x = max(new_x, x_min)
+                            new_x = min(new_x, int(i.parent_width - i.w + x_max))
+                            new_y = max(new_y, y_min)
+                            new_y = min(new_y, int(i.parent_height - i.h + y_max))
+                        else: # 2 arguments; (width, height)
+                            x_width, y_height = self.drag_offscreen
+                            new_x = max(new_x, int(x_width - i.w))
+                            new_x = min(new_x, int(i.parent_width - x_width))
+                            new_y = max(new_y, int(y_height - i.h))
+                            new_y = min(new_y, int(i.parent_height - y_height))
+
+                    # Callable called with x, y position
+                    elif callable(self.drag_offscreen):
+                        new_x, new_y = self.drag_offscreen(new_x, new_y)
+
                     if i.drag_group is not None and i.drag_name is not None:
                         i.drag_group.positions[i.drag_name] = (new_x, new_y, self.old_position)
 
@@ -849,6 +947,7 @@ class DragGroup(renpy.display.layout.MultiBox):
         super(DragGroup, self).add(child)
 
         self.sorted = False
+        renpy.display.render.invalidate(self)
 
     def remove(self, child):
         """
