@@ -1,4 +1,4 @@
-# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2023 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -65,9 +65,13 @@ def clear_capture_focus(name="default"):
     :doc: other
 
     Clear the captured focus with `name`.
+    If `name` is None, clear all captured focuses.
     """
 
-    focus_storage.pop(name, None)
+    if name is None:
+        focus_storage.clear()
+    else:
+        focus_storage.pop(name, None)
 
 
 def get_focus_rect(name="default"):
@@ -223,6 +227,8 @@ def get_mouse():
     if focused is None:
         return None
     else:
+        if isinstance(focused, renpy.display.behavior.Button): # this affects Button and all its subclasses (like Imagebutton)
+            return focused.style.mouse or "button" # prioritize button style over default keyword
         return focused.style.mouse
 
 
@@ -335,6 +341,7 @@ def before_interact(roots):
     global override
     global grab
     global modal_generation
+    global old_max_default
 
     modal_generation = 0
 
@@ -427,7 +434,9 @@ def before_interact(roots):
     # switch to the default.
     if should_max_default and (max_default > old_max_default):
         current = max_default_focus
+        set_grab(None)
         set_focused(max_default_focus, None, max_default_screen)
+        old_max_default = max_default
 
     # Try to find the current focus.
     if current is not None:
@@ -461,7 +470,7 @@ def before_interact(roots):
 
     # Finally, mark the current widget as the focused widget, and
     # all other widgets as unfocused.
-    for f, n, screen, modal in fwn:
+    for f, n, screen, _modal in fwn:
         if f is not current:
             renpy.display.screen.push_current_screen(screen)
             try:
@@ -537,9 +546,32 @@ def clear_focus():
     Clears the focus when the window loses mouse focus.
     """
 
+    set_grab(None)
     change_focus(None)
 
-# This handles mouse events, to see if they change the focus.
+
+def force_focus(d, arg=None):
+    """
+    Forces the focus to `d` immediately.
+    """
+
+    current = get_focused()
+
+    if (current is d) and (arg == argument):
+        return None
+
+    set_grab(None)
+
+    if current is not None:
+
+        try:
+            renpy.display.screen.push_current_screen(screen_of_focused)
+            current.unfocus(default=False)
+        finally:
+            renpy.display.screen.pop_current_screen()
+
+    set_focused(d, arg, renpy.display.screen.current_screen())
+    return d.focus(default=False)
 
 
 def mouse_handler(ev, x, y, default=False):
