@@ -1,4 +1,4 @@
-# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2023 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -71,7 +71,15 @@ class CacheEntry(object):
             rv += self.width * self.height
 
         if self.texture is not None:
-            rv += self.bounds[2] * self.bounds[3]
+
+            has_mipmaps = getattr(self.texture, "has_mipmaps", None)
+
+            if has_mipmaps and has_mipmaps():
+                mipmap_multiplier = 1.34
+            else:
+                mipmap_multiplier = 1.0
+
+            rv += int(self.bounds[2] * self.bounds[3] * mipmap_multiplier)
 
         return rv
 
@@ -115,9 +123,12 @@ class Cache(object):
         self.cache_limit = 0
 
         # The preload thread.
-        self.preload_thread = threading.Thread(target=self.preload_thread_main, name="preloader")
-        self.preload_thread.daemon = True
-        self.preload_thread.start()
+        if not renpy.emscripten:
+            self.preload_thread = threading.Thread(target=self.preload_thread_main, name="preloader")
+            self.preload_thread.daemon = True
+            self.preload_thread.start()
+        else:
+            self.preload_thread = None
 
         # Have we been added this tick?
         self.added = set()
@@ -181,6 +192,9 @@ class Cache(object):
             self.cache_limit = int(renpy.config.image_cache_size_mb * 1024 * 1024 // 4)
 
     def quit(self): # @ReservedAssignment
+        if not self.preload_thread:
+            return
+
         if not self.preload_thread.is_alive():
             return
 
@@ -248,6 +262,29 @@ class Cache(object):
     # generation of images.
     def get(self, image, predict=False, texture=False, render=False):
 
+        def make_render(ce):
+            bounds = ce.bounds[:2]
+
+            oversample = image.get_oversample() or .001
+
+            if oversample != 1:
+                inv_oversample = 1.0 / oversample
+
+                rv = renpy.display.render.Render(ce.width * inv_oversample, ce.height * inv_oversample)
+                rv.forward = renpy.display.matrix.Matrix2D(oversample, 0, 0, oversample)
+                rv.reverse = renpy.display.matrix.Matrix2D(inv_oversample, 0, 0, inv_oversample)
+
+                bounds = tuple(round(el / oversample) for el in bounds)
+            else:
+                rv = renpy.display.render.Render(ce.width, ce.height)
+
+            rv.blit(ce.texture, bounds)
+
+            if image.pixel_perfect:
+                rv.add_property("pixel_perfect", True)
+
+            return rv
+
         if render:
             texture = True
 
@@ -274,9 +311,7 @@ class Cache(object):
                     return None
 
                 if render:
-                    rv = renpy.display.render.Render(ce.width, ce.height)
-                    rv.blit(ce.texture, ce.bounds[:2])
-                    return rv
+                    return make_render(ce)
                 else:
                     return ce.texture
 
@@ -300,7 +335,11 @@ class Cache(object):
 
             if optimize_bounds:
                 bounds = tuple(surf.get_bounding_rect())
-                bounds = expands_bounds(bounds, size, renpy.config.expand_texture_bounds)
+                bounds = expand_bounds(bounds, size, renpy.config.expand_texture_bounds)
+
+                if image.oversample > 1:
+                    bounds = ensure_bounds_divide_evenly(bounds, image.oversample)
+
                 w = bounds[2]
                 h = bounds[3]
             else:
@@ -338,15 +377,16 @@ class Cache(object):
 
                 ce.texture = renpy.display.draw.load_texture(texsurf)
 
+                # This was loaded while predicting images for immediate use,
+                # so get it onto the GPU.
+                if not predict and renpy.display.draw is not None:
+                    while renpy.display.draw.ready_one_texture():
+                        pass
+
             if not predict:
-                if render:
-                    rv = renpy.display.render.Render(ce.width, ce.height)
-                    rv.blit(ce.texture, ce.bounds[:2])
-                else:
-                    rv = ce.texture
+                rv = ce.texture
             else:
                 rv = None
-
         else:
             rv = ce.surf
 
@@ -357,11 +397,14 @@ class Cache(object):
 
             ce.surf = None
 
+        if texture and render and not predict:
+            return make_render(ce)
+
         if (ce.surf is None) and (ce.texture is None):
             with self.lock:
                 self.kill(ce)
 
-        # Done... return the surface.
+        # Done. Return the surface or texture.
         return rv
 
     # This kills off a given cache entry.
@@ -588,6 +631,8 @@ class ImageBase(renpy.display.core.Displayable):
     __version__ = 1
 
     optimize_bounds = False
+    oversample = 1
+    pixel_perfect = False
 
     def after_upgrade(self, version):
         if version < 1:
@@ -598,6 +643,10 @@ class ImageBase(renpy.display.core.Displayable):
         self.rle = properties.pop('rle', None)
         self.cache = properties.pop('cache', True)
         self.optimize_bounds = properties.pop('optimize_bounds', True)
+        self.oversample = properties.pop('oversample', 1)
+
+        if self.oversample <= 0:
+            raise Exception("Image's oversample parameter must be greater than 0.")
 
         properties.setdefault('style', 'image')
 
@@ -645,9 +694,12 @@ class ImageBase(renpy.display.core.Displayable):
 
         return 0
 
+    def get_oversample(self):
+        """
+        Returns the oversample value for this image.
+        """
 
-
-
+        return self.oversample
 
 
 ignored_images = set()
@@ -659,30 +711,50 @@ class Image(ImageBase):
     This image manipulator loads an image from a file.
     """
 
+    is_svg = False
+
     def __init__(self, filename, **properties):
         """
         @param filename: The filename that the image will be loaded from.
         """
 
+        if "@" in filename:
+            base = filename.rpartition(".")[0]
+            extras = base.partition("@")[2].split(",")
+
+            for i in extras:
+                try:
+                    oversample = float(i)
+                    properties.setdefault('oversample', oversample)
+                except Exception:
+                    raise Exception("Unknown image modifier %r in %r." % (i, filename))
+
         super(Image, self).__init__(filename, **properties)
         self.filename = filename
 
+
     def _repr_info(self):
         return repr(self.filename)
 
     def get_hash(self):
         return renpy.loader.get_hash(self.filename)
 
+    def get_oversample(self):
+        if self.is_svg:
+            return self.oversample * renpy.display.draw.draw_per_virt
+        else:
+            return self.oversample
+
     def load(self, unscaled=False):
 
+        # Unscaled is no longer used.
+
         cache.add_load_log(self.filename)
 
-
         try:
 
-
             try:
-                filelike = renpy.loader.load(self.filename)
+                filelike = renpy.loader.load(self.filename, directory="images")
                 filename = self.filename
                 force_size = None
             except renpy.webloader.DownloadNeeded as e:
@@ -693,15 +765,26 @@ class Image(ImageBase):
                 force_size = e.size
 
             with filelike as f:
-                if unscaled:
-                    surf = renpy.display.pgrender.load_image_unscaled(f, filename)
-                else:
-                    surf = renpy.display.pgrender.load_image(f, filename)
+                surf = renpy.display.pgrender.load_image(f, filename)
 
             if force_size is not None:
                 # avoid size-related exceptions (e.g. Crop on a smaller placeholder)
                 surf = renpy.display.pgrender.transform_scale(surf, force_size)
 
+            self.is_svg = filename.lower().endswith(".svg")
+            self.pixel_perfect = self.is_svg
+
+            if self.is_svg:
+                width, height = surf.get_size()
+
+                width = int(width * renpy.display.draw.draw_per_virt)
+                height = int(height * renpy.display.draw.draw_per_virt)
+
+                filelike = renpy.loader.load(self.filename, directory="images")
+
+                with filelike as f:
+                    surf = renpy.display.pgrender.load_image(filelike, filename, size=(width, height))
+
             return surf
 
         except Exception as e:
@@ -721,11 +804,9 @@ class Image(ImageBase):
                 else:
                     return Image("_missing_image.png").load()
 
-            raise e
-
     def predict_files(self):
 
-        if renpy.loader.loadable(self.filename):
+        if renpy.loader.loadable(self.filename, directory="images"):
             return [ self.filename ]
         else:
             if renpy.config.missing_image_callback:
@@ -825,6 +906,9 @@ class Composite(ImageBase):
         self.positions = args[0::2]
         self.images = [ image(i) for i in args[1::2] ]
 
+        # Only supports all the images having the same oversample factor
+        self.oversample = self.images[0].get_oversample()
+
     def get_hash(self):
         rv = 0
 
@@ -840,10 +924,13 @@ class Composite(ImageBase):
         else:
             size = cache.get(self.images[0]).get_size()
 
+        os = self.oversample
+        size = [s*os for s in size]
+
         rv = renpy.display.pgrender.surface(size, True)
 
         for pos, im in zip(self.positions, self.images):
-            rv.blit(cache.get(im), pos)
+            rv.blit(cache.get(im), [p*os for p in pos])
 
         return rv
 
@@ -878,6 +965,7 @@ class Scale(ImageBase):
         super(Scale, self).__init__(im, width, height, bilinear, **properties)
 
         self.image = im
+        self.oversample = im.get_oversample()
         self.width = int(width)
         self.height = int(height)
         self.bilinear = bilinear
@@ -888,17 +976,18 @@ class Scale(ImageBase):
     def load(self):
 
         child = cache.get(self.image)
+        os = self.oversample
 
         if self.bilinear:
             try:
                 renpy.display.render.blit_lock.acquire()
-                rv = renpy.display.scale.smoothscale(child, (self.width, self.height))
+                rv = renpy.display.scale.smoothscale(child, (self.width*os, self.height*os))
             finally:
                 renpy.display.render.blit_lock.release()
         else:
             try:
                 renpy.display.render.blit_lock.acquire()
-                rv = renpy.display.pgrender.transform_scale(child, (self.width, self.height))
+                rv = renpy.display.pgrender.transform_scale(child, (self.width*os, self.height*os))
             finally:
                 renpy.display.render.blit_lock.release()
 
@@ -936,6 +1025,7 @@ class FactorScale(ImageBase):
         super(FactorScale, self).__init__(im, width, height, bilinear, **properties)
 
         self.image = im
+        self.oversample = im.get_oversample()
         self.width = width
         self.height = height
         self.bilinear = bilinear
@@ -997,6 +1087,7 @@ class Flip(ImageBase):
         super(Flip, self).__init__(im, horizontal, vertical, **properties)
 
         self.image = im
+        self.oversample = im.get_oversample()
         self.horizontal = horizontal
         self.vertical = vertical
 
@@ -1039,6 +1130,7 @@ class Rotozoom(ImageBase):
         super(Rotozoom, self).__init__(im, angle, zoom, **properties)
 
         self.image = im
+        self.oversample = im.get_oversample()
         self.angle = angle
         self.zoom = zoom
 
@@ -1086,6 +1178,7 @@ class Crop(ImageBase):
         super(Crop, self).__init__(im, x, y, w, h, **properties)
 
         self.image = im
+        self.oversample = im.get_oversample()
         self.x = x
         self.y = y
         self.w = w
@@ -1095,8 +1188,9 @@ class Crop(ImageBase):
         return self.image.get_hash()
 
     def load(self):
-        return cache.get(self.image).subsurface((self.x, self.y,
-                                                 self.w, self.h))
+        os = self.oversample
+        return cache.get(self.image).subsurface((self.x*os, self.y*os,
+                                                 self.w*os, self.h*os))
 
     def predict_files(self):
         return self.image.predict_files()
@@ -1146,6 +1240,7 @@ class Map(ImageBase):
         super(Map, self).__init__(im, rmap, gmap, bmap, amap, force_alpha, **properties)
 
         self.image = im
+        self.oversample = im.get_oversample()
         self.rmap = rmap
         self.gmap = gmap
         self.bmap = bmap
@@ -1191,6 +1286,7 @@ class Twocolor(ImageBase):
         super(Twocolor, self).__init__(im, white, black, force_alpha, **properties)
 
         self.image = im
+        self.oversample = im.get_oversample()
         self.white = white
         self.black = black
 
@@ -1229,6 +1325,7 @@ class Recolor(ImageBase):
         super(Recolor, self).__init__(im, rmul, gmul, bmul, amul, force_alpha, **properties)
 
         self.image = im
+        self.oversample = im.get_oversample()
         self.rmul = rmul + 1
         self.gmul = gmul + 1
         self.bmul = bmul + 1
@@ -1278,6 +1375,7 @@ class Blur(ImageBase):
         super(Blur, self).__init__(im, xrad, yrad, **properties)
 
         self.image = im
+        self.oversample = im.get_oversample()
         self.rx = xrad
         self.ry = xrad if yrad is None else yrad
 
@@ -1291,7 +1389,7 @@ class Blur(ImageBase):
         ws = renpy.display.pgrender.surface(surf.get_size(), True)
         rv = renpy.display.pgrender.surface(surf.get_size(), True)
 
-        renpy.display.module.blur(surf, ws, rv, self.rx, self.ry)
+        renpy.display.module.blur(surf, ws, rv, self.rx*self.oversample, self.ry*self.oversample)
 
         return rv
 
@@ -1342,6 +1440,7 @@ class MatrixColor(ImageBase):
         super(MatrixColor, self).__init__(im, matrix, **properties)
 
         self.image = im
+        self.oversample = im.get_oversample()
         self.matrix = matrix
 
     def get_hash(self):
@@ -1765,6 +1864,7 @@ class Tile(ImageBase):
 
         super(Tile, self).__init__(im, size, **properties)
         self.image = im
+        self.oversample = im.get_oversample()
         self.size = size
 
     def get_hash(self):
@@ -1777,6 +1877,10 @@ class Tile(ImageBase):
         if size is None:
             size = (renpy.config.screen_width, renpy.config.screen_height)
 
+        os = self.oversample
+
+        size = [round(v*os) for v in size]
+
         surf = cache.get(self.image)
 
         rv = renpy.display.pgrender.surface(size, True)
@@ -1809,6 +1913,8 @@ class AlphaMask(ImageBase):
 
     Note that this takes different arguments from :func:`AlphaMask`,
     which uses the mask's alpha channel.
+
+    The two images need to have the same size, and the same oversampling factor.
     """
 
     def __init__(self, base, mask, **properties):
@@ -1817,6 +1923,9 @@ class AlphaMask(ImageBase):
         self.base = image(base)
         self.mask = image(mask)
 
+        # The two images already need to be the same size, they now also need the same oversample.
+        self.oversample = self.base.get_oversample()
+
     def get_hash(self):
         return self.base.get_hash() + self.mask.get_hash()
 
@@ -1842,18 +1951,25 @@ def image(arg, loose=False, **properties):
     """
     :doc: im_image
     :name: Image
-    :args: (filename, *, optimize_bounds=True, **properties)
+    :args: (filename, *, optimize_bounds=True, oversample=1, **properties)
 
     Loads an image from a file. `filename` is a
     string giving the name of the file.
 
-    `filename` should be a JPEG or PNG file with an appropriate
-    extension.
+    `filename`
+        This should be an image filename, including the extension.
 
-    If optimize_bounds is True, only the portion of the image that
-    inside the bounding box of non-transparent pixels is loaded into
-    GPU memory. (The only reason to set this to False is when using an
-    image as input to a shader.)
+    `optimize_bounds`
+        If true, only the portion of the image that
+        inside the bounding box of non-transparent pixels is loaded into
+        GPU memory. (The only reason to set this to False is when using an
+        image as input to a shader.)
+
+    `oversample`
+        If this is greater than 1, the image is considered to be oversampled,
+        with more pixels than its logical size would imply. For example, if
+        an image file is 2048x2048 and oversample is 2, then the image will
+        be treated as a 1024x1024 image for the purpose of layout.
     """
 
     """
@@ -1894,7 +2010,7 @@ def image(arg, loose=False, **properties):
         raise Exception("Could not construct image from argument.")
 
 
-def expands_bounds(bounds, size, amount):
+def expand_bounds(bounds, size, amount):
     """
     This expands the rectangle bounds by amount, while ensure it fits inside size.
     """
@@ -1910,6 +2026,28 @@ def expands_bounds(bounds, size, amount):
     return (x0, y0, x1 - x0, y1 - y0)
 
 
+def ensure_bounds_divide_evenly(bounds, n):
+    """
+    This ensures that the bounds is divisible by n, by expanding the bounds
+    if necessary.
+    """
+
+    x, y, w, h = bounds
+
+    xmodulo = x % n
+    ymodulo = y % n
+
+    if xmodulo:
+        x -= xmodulo
+        w += xmodulo
+
+    if ymodulo:
+        y -= ymodulo
+        h += ymodulo
+
+    return (x, y, w, h)
+
+
 def load_image(im):
     """
     :name: renpy.load_image
@@ -1931,6 +2069,21 @@ def load_surface(im):
 
     return cache.get(image(im))
 
+def load_rgba(data, size):
+    """
+    :name: renpy.load_rgba
+    :doc: udd_utility
+
+    Loads the image data `bytes` into a texture of size `size`, and return it.
+
+    `data`
+        Should be a bytes object containing the image data in RGBA8888 order.
+    """
+
+    surf = renpy.display.pgrender.surface(size, True)
+    surf.from_data(data)
+    return renpy.display.draw.load_texture(surf)
+
 
 def reset_module():
     print("Resetting cache.")
