@@ -1,4 +1,4 @@
-# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2023 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -24,13 +24,14 @@
 # are None, at least to the point of making it through __init__. This is
 # so that prediction of images works.
 
-from __future__ import print_function
+from __future__ import division, absolute_import, with_statement, print_function, unicode_literals
+from renpy.compat import PY2, basestring, bchr, bord, chr, open, pystr, range, round, str, tobytes, unicode # *
 
-import renpy.display
+
+
+import renpy
 
 # Utility function used by MoveTransition et al.
-
-
 def position(d):
 
     xpos, ypos, xanchor, yanchor, _xoffset, _yoffset, _subpixel = d.get_placement()
@@ -82,7 +83,7 @@ def MoveIn(pos, pos1, delay, d, **kwargs):
             return b
         return a
 
-    pos = tuple([aorb(a, b) for a, b in zip(pos, pos1)])
+    pos = tuple(aorb(a, b) for a, b in zip(pos, pos1))
     return renpy.display.motion.Move(pos, pos1, delay, d, **kwargs)
 
 
@@ -93,7 +94,7 @@ def MoveOut(pos, pos1, delay, d, **kwargs):
             return b
         return a
 
-    pos = tuple([aorb(a, b) for a, b in zip(pos, pos1)])
+    pos = tuple(aorb(a, b) for a, b in zip(pos, pos1))
     return renpy.display.motion.Move(pos1, pos, delay, d, **kwargs)
 
 
@@ -234,18 +235,18 @@ def OldMoveTransition(delay, old_widget=None, new_widget=None, factory=None, ent
 
         def merge(sle, d):
             rv = sle.copy()
-            rv.show_time = 0
+            rv.show_time = None
             rv.displayable = d
             return rv
 
         def entering(sle):
-            new_d = wrap(new_sle)
+            new_d = wrap(sle)
             move = enter_factory(position(new_d), delay, new_d, **offsets(new_d))
 
             if move is None:
                 return
 
-            rv_sl.append(merge(new_sle, move))
+            rv_sl.append(merge(sle, move))
 
         def leaving(sle):
             old_d = wrap(sle)
@@ -255,7 +256,7 @@ def OldMoveTransition(delay, old_widget=None, new_widget=None, factory=None, ent
                 return
 
             move = renpy.display.layout.IgnoresEvents(move)
-            rv_sl.append(merge(old_sle, move))
+            rv_sl.append(merge(sle, move))
 
         def moving(old_sle, new_sle):
             old_d = wrap(old_sle)
@@ -336,7 +337,7 @@ def OldMoveTransition(delay, old_widget=None, new_widget=None, factory=None, ent
     # This calls merge_slide to actually do the merging.
 
     rv = merge_slide(old_widget, new_widget)
-    rv.delay = delay  # W0201
+    rv.delay = delay # W0201
 
     return rv
 
@@ -418,10 +419,10 @@ class MoveInterpolate(renpy.display.core.Displayable):
 
     def get_placement(self):
 
-        if self.st > self.delay:
+        if self.st > self.delay: # type: ignore
             done = 1.0
         else:
-            done = self.st / self.delay
+            done = self.st / self.delay # type: ignore
 
         if self.time_warp is not None:
             done = self.time_warp(done)
@@ -448,33 +449,40 @@ class MoveInterpolate(renpy.display.core.Displayable):
 def MoveTransition(delay, old_widget=None, new_widget=None, enter=None, leave=None, old=False, layers=[ 'master' ], time_warp=None, enter_time_warp=None, leave_time_warp=None):
     """
     :doc: transition function
-    :args: (delay, enter=None, leave=None, old=False, layers=['master'], time_warp=None, enter_time_warp=None, leave_time_warp=None)
+    :args: (delay, *, enter=None, leave=None, old=False, layers=['master'], time_warp=None, enter_time_warp=None, leave_time_warp=None)
     :name: MoveTransition
 
     Returns a transition that interpolates the position of images (with the
     same tag) in the old and new scenes.
 
+    As only layers have tags, MoveTransitions can only be applied to a single
+    layer or all layers at once, using the with statement. It will not work
+    in other contexts, like ATL, :func:`ComposeTransition`, or other ways of
+    applying transitions.
+
     `delay`
         The time it takes for the interpolation to finish.
 
     `enter`
         If not None, images entering the scene will also be moved. The value
         of `enter` should be a transform that is applied to the image to
-        get its starting position.
+        get it in its starting position.
 
     `leave`
-        If not None, images leaving the scene will also be move. The value
+        If not None, images leaving the scene will also be moved. The value
         of `leave` should be a transform that is applied to the image to
-        get its ending position.
+        get it in its ending position.
 
     `old`
-        If true, the old image will be used in preference to the new one.
+        If true, when a tag gets its image changed during the transition,
+        the old image will be used in preference to the new one. Otherwise,
+        the new images will be used.
 
     `layers`
         A list of layers that moves are applied to.
 
     `time_warp`
-        A time warp function that's applied to the interpolation. This
+        A :ref:`time warp function <warpers>` that's applied to the interpolation. This
         takes a number between 0.0 and 1.0, and should return a number in
         the same range.
 
@@ -483,12 +491,23 @@ def MoveTransition(delay, old_widget=None, new_widget=None, enter=None, leave=No
 
     `leave_time_warp`
         A time warp function that's applied to images leaving the scene.
-
     """
 
+    if not (hasattr(old_widget, 'scene_list') or hasattr(old_widget, 'layers')):
+        if renpy.config.developer:
+            raise Exception("MoveTransition can only be applied to one or all layers, not %s." % type(old_widget).__name__)
+
+    if not (hasattr(new_widget, 'scene_list') or hasattr(new_widget, 'layers')):
+        if renpy.config.developer:
+            raise Exception("MoveTransition can only be applied to one or all layers, not %s." % type(new_widget).__name__)
+
     use_old = old
 
-    def merge_slide(old, new):
+    def merge_slide(old, new, merge_slide):
+
+        # This function takes itself as an argument to prevent a reference
+        # loop that occurs when it refers to itself in the it's parent's
+        # scope.
 
         # If new does not have .layers or .scene_list, then we simply
         # insert a move from the old position to the new position, if
@@ -508,17 +527,40 @@ def MoveTransition(delay, old_widget=None, new_widget=None, enter=None, leave=No
 
             rv = renpy.display.layout.MultiBox(layout='fixed')
 
+            rv.layers = { }
+
             for layer in renpy.config.layers:
 
-                f = new.layers[layer]
+                d = merge_slide(old.layers[layer], new.layers[layer], merge_slide)
 
-                if (isinstance(f, renpy.display.layout.MultiBox)
-                    and layer in layers
-                        and f.scene_list is not None):
+                rv.layers[layer] = d
+                rv.add(d, True, True)
 
-                    f = merge_slide(old.layers[layer], new.layers[layer])
+            return rv
 
-                rv.add(f)
+        # Unpack old if needs be.
+        old = old.untransformed_layer or old
+
+        # If we're dealing with a wrapped layer widget, merge the raw
+        # children then re-apply the transform.
+        if new.untransformed_layer:
+            rv = new
+
+            new = new.untransformed_layer
+            layer = new.layer_name
+
+            if (isinstance(new, renpy.display.layout.MultiBox) and
+                layer in layers and new.scene_list is not None):
+
+                d = merge_slide(old, new, merge_slide)
+
+                adjust = renpy.display.layout.AdjustTimes(d, None, None)
+                rv = renpy.game.context().scene_lists.transform_layer(layer, adjust)
+
+                if rv is adjust:
+                    rv = d
+                else:
+                    rv = renpy.display.layout.MatchTimes(rv, adjust)
 
             return rv
 
@@ -544,9 +586,9 @@ def MoveTransition(delay, old_widget=None, new_widget=None, enter=None, leave=No
             if not enter:
                 return
 
-            new_d = wrap(new_sle)
-            move = MoveInterpolate(delay, enter(new_d), new_d, False, enter_time_warp)
-            rv_sl.append(merge(new_sle, move))
+            new_d = wrap(sle)
+            move = MoveInterpolate(delay, renpy.store.At(new_d, enter), new_d, False, enter_time_warp)
+            rv_sl.append(merge(sle, move))
 
         def leaving(sle):
 
@@ -554,9 +596,9 @@ def MoveTransition(delay, old_widget=None, new_widget=None, enter=None, leave=No
                 return
 
             old_d = wrap(sle)
-            move = MoveInterpolate(delay, old_d, leave(old_d), True, leave_time_warp)
+            move = MoveInterpolate(delay, old_d, renpy.store.At(old_d, leave), True, leave_time_warp)
             move = renpy.display.layout.IgnoresEvents(move)
-            rv_sl.append(merge(old_sle, move))
+            rv_sl.append(merge(sle, move))
 
         def moving(old_sle, new_sle):
 
@@ -632,7 +674,7 @@ def MoveTransition(delay, old_widget=None, new_widget=None, enter=None, leave=No
         return rv
 
     # Call merge_slide to actually do the merging.
-    rv = merge_slide(old_widget, new_widget)
+    rv = merge_slide(old_widget, new_widget, merge_slide)
     rv.delay = delay
 
     return rv
