@@ -1,4 +1,4 @@
-# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2023 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -31,8 +31,24 @@ import types # @UnresolvedImport
 
 import renpy
 from renpy.display.layout import Container
-from renpy.display.accelerator import transform_render
+from renpy.display.accelerator import RenderTransform
 from renpy.atl import position, any_object, bool_or_none, float_or_none, matrix, mesh
+from renpy.display.core import absolute
+
+class Camera(renpy.object.Object):
+    """
+    :doc: point_to_camera
+
+    Instances of this class can be used with point_to to point
+    at the location of the camera for a particular layer.
+
+    `layer`
+        The name of the layer.
+    """
+
+    def __init__(self, layer="master"):
+        self.layer = layer
+
 
 # The null object that's used if we don't have a defined child.
 null = None
@@ -47,42 +63,6 @@ def get_null():
 
     return null
 
-# Convert a position from cartesian to polar coordinates.
-
-
-def cartesian_to_polar(x, y, xaround, yaround):
-    """
-    Converts cartesian coordinates to polar coordinates.
-    """
-
-    dx = x - xaround
-    dy = y - yaround
-
-    radius = math.hypot(dx, dy)
-    angle = math.atan2(dx, -dy) / math.pi * 180
-
-    if angle < 0:
-        angle += 360
-
-    return angle, radius
-
-
-def polar_to_cartesian(angle, radius, xaround, yaround):
-    """
-    Converts polart coordinates to cartesian coordinates.
-    """
-
-    angle = angle * math.pi / 180
-
-    dx = radius * math.sin(angle)
-    dy = -radius * math.cos(angle)
-
-    x = type(xaround)(xaround + dx)
-    y = type(yaround)(yaround + dy)
-
-    return x, y
-
-
 def first_not_none(*args):
     """
     Returns the first argument that is not None, or the last argument if
@@ -98,9 +78,15 @@ def first_not_none(*args):
 
 class TransformState(renpy.object.Object):
 
-    last_angle = None
+    last_angle = 0.0
+    last_anchorangle = 0.0
     last_events = True
 
+    available_width = 0
+    available_height = 0
+
+    radius_type = absolute
+
     def __init__(self):
 
         # Most fields on this object are set by add_property, at the bottom
@@ -127,8 +113,12 @@ class TransformState(renpy.object.Object):
             d[k] = getattr(ts, k)
 
         self.last_angle = ts.last_angle
+        self.last_anchorangle = ts.last_anchorangle
         self.last_events = ts.last_events
 
+        self.available_width = ts.available_width
+        self.available_height = ts.available_height
+
         # Set the position and anchor to None, so inheritance works.
         if self.perspective is None: # type: ignore
             self.xpos = None
@@ -220,14 +210,114 @@ class TransformState(renpy.object.Object):
 
     yalign = property(get_yalign, set_yalign)
 
+    def scale(self, value, available):
+        """
+        Converts value to a float, scaled by the available area, if
+        required.
+        """
+
+        if type(value) is float:
+            return value * available
+
+        return 1.0 * value
+
+    def cartesian_to_polar_pos(self, x, y):
+        """
+        Converts cartesian coordinates to polar coordinates.
+        """
+
+        x = self.scale(x, self.available_width)
+        y = self.scale(y, self.available_height)
+
+        xaround = self.scale(self.xaround, self.available_width)
+        yaround = self.scale(self.yaround, self.available_height)
+
+        dx = x - xaround
+        dy = y - yaround
+
+        radius = math.hypot(dx, dy)
+        angle = math.atan2(dx, -dy) / math.pi * 180
+
+        if angle < 0:
+            angle += 360
+
+        return angle, radius
+
+
+    def polar_to_cartesian_pos(self, angle, radius):
+        """
+        Converts polar coordinates to cartesian coordinates.
+        """
+
+        xaround = self.scale(self.xaround, self.available_width)
+        yaround = self.scale(self.yaround, self.available_height)
+
+        angle = angle * math.pi / 180
+
+        dx = radius * math.sin(angle)
+        dy = -radius * math.cos(angle)
+
+        x = absolute(xaround + dx)
+        y = absolute(yaround + dy)
+
+        xpos = first_not_none(self.xpos, self.inherited_xpos, 0)
+        ypos = first_not_none(self.ypos, self.inherited_ypos, 0)
+
+        if type(xpos) is float:
+            x = float(x / self.available_width)
+
+        if type(ypos) is float:
+            y = float(y / self.available_height)
+
+        return x, y
+
+    def cartesian_to_polar_anchor(self, x, y):
+        """
+        Converts cartesian coordinates to polar coordinates.
+        """
+
+        xaround = self.xanchoraround
+        yaround = self.yanchoraround
+
+        dx = x - xaround
+        dy = y - yaround
+
+        radius = math.hypot(dx, dy)
+        angle = math.atan2(dx, -dy) / math.pi * 180
+
+        if angle < 0:
+            angle += 360
+
+        return angle, type(xaround)(radius)
+
+    def polar_to_cartesian_anchor(self, angle, radius):
+        """
+        Converts polar coordinates to cartesian coordinates.
+        Assumes that xanchoraround, yanchoraround, and radius
+        are all in the same units.
+        """
+
+        xaround = self.xanchoraround
+        yaround = self.yanchoraround
+
+        angle = angle * math.pi / 180
+
+        dx = radius * math.sin(angle)
+        dy = -radius * math.cos(angle)
+
+        x = type(xaround)(xaround + dx)
+        y = type(xaround)(yaround + dy)
+
+        return x, y
+
     def get_around(self):
         return (self.xaround, self.yaround)
 
     def set_around(self, value):
         self.xaround, self.yaround = value
-        self.xanchoraround, self.yanchoraround = None, None
 
     def set_alignaround(self, value):
+        self.xanchor, self.yanchor = value
         self.xaround, self.yaround = value
         self.xanchoraround, self.yanchoraround = value
 
@@ -237,38 +327,87 @@ class TransformState(renpy.object.Object):
     def get_angle(self):
         xpos = first_not_none(self.xpos, self.inherited_xpos, 0)
         ypos = first_not_none(self.ypos, self.inherited_ypos, 0)
-        angle, _radius = cartesian_to_polar(xpos, ypos, self.xaround, self.yaround)
+        angle, _radius = self.cartesian_to_polar_pos(xpos, ypos)
         return angle
 
     def get_radius(self):
         xpos = first_not_none(self.xpos, self.inherited_xpos, 0)
         ypos = first_not_none(self.ypos, self.inherited_ypos, 0)
-        _angle, radius = cartesian_to_polar(xpos, ypos, self.xaround, self.yaround)
-        return radius
+        _angle, radius = self.cartesian_to_polar_pos(xpos, ypos)
+
+        if self.radius_type is float:
+            divisor = min(self.available_width, self.available_height)
+            if divisor:
+                radius = radius / divisor
+            else:
+                radius = 0.0
+
+        return self.radius_type(radius)
+
+    def set_angle(self, angle):
+        self.last_angle = angle
 
-    def set_angle(self, value):
         xpos = first_not_none(self.xpos, self.inherited_xpos, 0)
         ypos = first_not_none(self.ypos, self.inherited_ypos, 0)
-        _angle, radius = cartesian_to_polar(xpos, ypos, self.xaround, self.yaround)
-        angle = value
-        self.xpos, self.ypos = polar_to_cartesian(angle, radius, self.xaround, self.yaround)
+        _angle, radius = self.cartesian_to_polar_pos(xpos, ypos)
+        self.xpos, self.ypos = self.polar_to_cartesian_pos(angle, radius)
 
-        if self.xanchoraround:
-            self.xanchor, self.yanchor = polar_to_cartesian(angle, radius, self.xaround, self.yaround)
+    def set_radius(self, radius):
+        self.radius_type = type(radius)
+
+        if type(radius) is float:
+            radius = self.scale(radius, min(self.available_width, self.available_height))
 
-    def set_radius(self, value):
         xpos = first_not_none(self.xpos, self.inherited_xpos, 0)
         ypos = first_not_none(self.ypos, self.inherited_ypos, 0)
-        angle, _radius = cartesian_to_polar(xpos, ypos, self.xaround, self.yaround)
-        radius = value
-        self.xpos, self.ypos = polar_to_cartesian(angle, radius, self.xaround, self.yaround)
+        angle, old_radius = self.cartesian_to_polar_pos(xpos, ypos)
 
-        if self.xanchoraround:
-            self.xanchor, self.yanchor = polar_to_cartesian(angle, radius, self.xaround, self.yaround)
+        # Deal with the angle becoming 0.0 when the radius is 0.0.
+        if not old_radius and self.last_angle is not None:
+            angle = self.last_angle
+
+        self.xpos, self.ypos = self.polar_to_cartesian_pos(angle, radius)
 
     angle = property(get_angle, set_angle)
     radius = property(get_radius, set_radius)
 
+    # Anchor polar motions.
+
+    def get_anchorangle(self):
+        xanchor = first_not_none(self.xanchor, self.inherited_xanchor, 0)
+        yanchor = first_not_none(self.yanchor, self.inherited_yanchor, 0)
+        angle, _radius = self.cartesian_to_polar_anchor(xanchor, yanchor)
+        return angle
+
+    def get_anchorradius(self):
+        xanchor = first_not_none(self.xanchor, self.inherited_xanchor, 0)
+        yanchor = first_not_none(self.yanchor, self.inherited_yanchor, 0)
+        _angle, radius = self.cartesian_to_polar_anchor(xanchor, yanchor)
+
+        return self.radius_type(radius)
+
+    def set_anchorangle(self, angle):
+        self.last_anchorangle = angle
+
+        xanchor = first_not_none(self.xanchor, self.inherited_xanchor, 0)
+        yanchor = first_not_none(self.yanchor, self.inherited_yanchor, 0)
+        _angle, radius = self.cartesian_to_polar_anchor(xanchor, yanchor)
+        self.xanchor, self.yanchor = self.polar_to_cartesian_anchor(angle, radius)
+
+    def set_anchorradius(self, radius):
+        xanchor = first_not_none(self.xanchor, self.inherited_xanchor, 0)
+        yanchor = first_not_none(self.yanchor, self.inherited_yanchor, 0)
+        angle, old_radius = self.cartesian_to_polar_anchor(xanchor, yanchor)
+
+        # Deal with the angle becoming 0.0 when the radius is 0.0.
+        if not old_radius and self.last_anchorangle is not None:
+            angle = self.last_anchorangle
+
+        self.xanchor, self.yanchor = self.polar_to_cartesian_anchor(angle, radius)
+
+    anchorangle = property(get_anchorangle, set_anchorangle)
+    anchorradius = property(get_anchorradius, set_anchorradius)
+
     def get_pos(self):
         return self.xpos, self.ypos
 
@@ -433,14 +572,15 @@ class Transform(Container):
                  focus=None,
                  default=False,
                  _args=None,
-                 alt=None,
 
                  **kwargs):
 
+        properties = {k: kwargs.pop(k) for k in style_properties if k in kwargs}
+
         self.kwargs = kwargs
         self.style_arg = style
 
-        super(Transform, self).__init__(style=style, focus=focus, default=default, _args=_args, alt=alt)
+        super(Transform, self).__init__(style=style, focus=focus, default=default, _args=_args, **properties)
 
         self.function = function
 
@@ -642,6 +782,12 @@ class Transform(Container):
 
     def _hide(self, st, at, kind):
 
+        if kind == "cancel":
+            if self.state.show_cancels_hide:
+                return None
+            else:
+                return self
+
         # Prevent time from ticking backwards, as can happen if we replace a
         # transform but keep its state.
         if st + self.st_offset <= self.st:
@@ -738,13 +884,29 @@ class Transform(Container):
         self.active = True
 
         if self.state.last_events != self.state.events:
-            if self.state.events:
+            if self.state.events and renpy.game.interface is not None:
                 renpy.game.interface.timeout(0)
             self.state.last_events = self.state.events
 
-    # The render method is now defined in accelerator.pyx.
     def render(self, width, height, st, at):
-        return transform_render(self, width, height, st, at)
+
+        # Prevent time from ticking backwards, as can happen if we replace a
+        # transform but keep its state.
+        if st + self.st_offset <= self.st:
+            self.st_offset = self.st - st
+        if at + self.at_offset <= self.at:
+            self.at_offset = self.at - at
+
+        self.st = st = st + self.st_offset
+        self.at = at = at + self.at_offset
+
+        self.state.available_width = width
+        self.state.available_height = height
+
+        # Update the state.
+        self.update_state()
+
+        return RenderTransform(self).render(width, height, st, at)
 
     def event(self, ev, x, y, st):
 
@@ -932,7 +1094,7 @@ class ATLTransform(renpy.atl.ATLTransformBase, Transform):
         self.active = True
 
         if self.state.last_events != self.state.events:
-            if self.state.events:
+            if self.state.events and renpy.game.interface is not None:
                 renpy.game.interface.timeout(0)
             self.state.last_events = self.state.events
 
@@ -941,6 +1103,9 @@ class ATLTransform(renpy.atl.ATLTransformBase, Transform):
         return repr((self.child, self.atl.loc))
 
 
+# Names of style properties that should be sent to the parent.
+style_properties = {'alt'}
+
 # Names of transform properties, and if the property should be handled with
 # diff2 or diff4.
 all_properties = set()
@@ -1019,12 +1184,18 @@ add_property("nearest", bool_or_none, None)
 add_property("perspective", any_object, None)
 add_property("rotate", float, None)
 add_property("rotate_pad", bool, True)
+add_property("point_to", any_object, None)
+add_property("orientation", (float, float, float), None)
+add_property("xrotate", float, None)
+add_property("yrotate", float, None)
+add_property("zrotate", float, None)
 add_property("shader", any_object, None, diff=None)
+add_property("show_cancels_hide", bool, True)
 add_property("subpixel", bool, False)
 add_property("transform_anchor", bool, False)
 add_property("zoom", float, 1.0)
 
-add_property("xanchoraround", float, 0.0)
+add_property("xanchoraround", float, 0.5)
 add_property("xanchor", position, None, diff=4)
 add_property("xaround", position, 0.0)
 add_property("xoffset", float, 0.0)
@@ -1034,7 +1205,7 @@ add_property("xsize", position, None)
 add_property("xtile", int, 1)
 add_property("xzoom", float, 1.0)
 
-add_property("yanchoraround", float, 0.0)
+add_property("yanchoraround", float, 0.5)
 add_property("yanchor", position, None, diff=4)
 add_property("yaround", position, 0.0)
 add_property("yoffset", float, 0.0)
@@ -1061,11 +1232,13 @@ ALIASES = {
     "alignaround" : (float, float),
     "align" : (float, float),
     "anchor" : (position, position),
+    "anchorangle" : float,
+    "anchorradius" : position,
     "angle" : float,
     "around" : (position, position),
     "offset" : (int, int),
     "pos" : (position, position),
-    "radius" : float,
+    "radius" : position,
     "size" : (int, int),
     "xalign" : float,
     "xcenter" : position,
