@@ -1,4 +1,4 @@
-# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2023 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -22,8 +22,14 @@
 # This file contains classes that handle layout of displayables on
 # the screen.
 
-import renpy.display
+from __future__ import division, absolute_import, with_statement, print_function, unicode_literals
+from renpy.compat import PY2, basestring, bchr, bord, chr, open, pystr, range, round, str, tobytes, unicode # *
+
+
+import math
+
 import pygame_sdl2 as pygame
+import renpy
 
 
 def edgescroll_proportional(n):
@@ -41,6 +47,10 @@ class Viewport(renpy.display.layout.Container):
     arrowkeys = False
     pagekeys = False
 
+    _draggable = True
+
+    drag_position_time = None
+
     def after_upgrade(self, version):
         if version < 1:
             self.xadjustment = renpy.display.behavior.Adjustment(1, 0)
@@ -62,14 +72,6 @@ class Viewport(renpy.display.layout.Container):
             self.edge_yspeed = 0
             self.edge_last_st = None
 
-        if version < 4:
-            self.xadjustment_param = None
-            self.yadjustment_param = None
-            self.offsets_param = (None, None)
-            self.set_adjustments_param = True
-            self.xinitial_param = None
-            self.yinitial_param = None
-
         if version < 5:
             self.focusable = self.draggable
 
@@ -92,33 +94,56 @@ class Viewport(renpy.display.layout.Container):
                  **properties):
 
         super(Viewport, self).__init__(style=style, **properties)
+
         if child is not None:
             self.add(child)
 
-        self.xadjustment_param = xadjustment
-        self.yadjustment_param = yadjustment
-        self.offsets_param = offsets
-        self.set_adjustments_param = set_adjustments
-        self.xinitial_param = xinitial
-        self.yinitial_param = yinitial
+        if xadjustment is None:
+            self.xadjustment = renpy.display.behavior.Adjustment(1, 0)
+        else:
+            self.xadjustment = xadjustment
 
-        self._show()
+        if yadjustment is None:
+            self.yadjustment = renpy.display.behavior.Adjustment(1, 0)
+        else:
+            self.yadjustment = yadjustment
+
+        if self.xadjustment.adjustable is None:
+            self.xadjustment.adjustable = True
+
+        if self.yadjustment.adjustable is None:
+            self.yadjustment.adjustable = True
+
+        self.set_adjustments = set_adjustments
+
+        self.xoffset = offsets[0] if (offsets[0] is not None) else xinitial
+        self.yoffset = offsets[1] if (offsets[1] is not None) else yinitial
+
+        if isinstance(replaces, Viewport) and replaces.offsets:
+            self.xadjustment.viewport_replaces(replaces.xadjustment)
+            self.yadjustment.viewport_replaces(replaces.yadjustment)
 
-        if isinstance(replaces, Viewport):
             self.xadjustment.range = replaces.xadjustment.range
-            self.yadjustment.range = replaces.yadjustment.range
             self.xadjustment.value = replaces.xadjustment.value
+            self.yadjustment.range = replaces.yadjustment.range
             self.yadjustment.value = replaces.yadjustment.value
             self.xoffset = replaces.xoffset
             self.yoffset = replaces.yoffset
             self.drag_position = replaces.drag_position
+            self.drag_position_time = replaces.drag_position_time
+            self.drag_speed = replaces.drag_speed
         else:
-            self.drag_position = None
+            self.drag_position = None # type: tuple[int, int]|None
+            self.drag_position_time = None # type: float|None
+            self.drag_speed = None
 
         self.child_width, self.child_height = child_size
 
+        if isinstance(draggable, bool):
+            self.draggable = draggable
+        else:
+            self.draggable = renpy.exports.variant(draggable)
         self.mousewheel = mousewheel
-        self.draggable = draggable
         self.arrowkeys = arrowkeys
         self.pagekeys = pagekeys
 
@@ -154,33 +179,17 @@ class Viewport(renpy.display.layout.Container):
             self.edge_speed = 0
             self.edge_function = edgescroll_proportional
 
-    def _show(self):
-        if self.xadjustment_param is None:
-            self.xadjustment = renpy.display.behavior.Adjustment(1, 0)
-        else:
-            self.xadjustment = self.xadjustment_param
-
-        if self.yadjustment_param is None:
-            self.yadjustment = renpy.display.behavior.Adjustment(1, 0)
-        else:
-            self.yadjustment = self.yadjustment_param
-
-        if self.xadjustment.adjustable is None:
-            self.xadjustment.adjustable = True
-
-        if self.yadjustment.adjustable is None:
-            self.yadjustment.adjustable = True
-
-        self.set_adjustments = self.set_adjustments_param
-
-        offsets = self.offsets_param
-        self.xoffset = offsets[0] if (offsets[0] is not None) else self.xinitial_param
-        self.yoffset = offsets[1] if (offsets[1] is not None) else self.yinitial_param
-
     def per_interact(self):
         self.xadjustment.register(self)
         self.yadjustment.register(self)
 
+    def set_style_prefix(self, prefix, root):
+        """
+        Do not change the style of children when the viewport is focused.
+        """
+
+        return
+
     def update_offsets(self, cw, ch, st):
         """
         This is called by render once we know the width (`cw`) and height (`ch`)
@@ -192,19 +201,24 @@ class Viewport(renpy.display.layout.Container):
         The returned offsets will be negative or zero.
         """
 
+        cw = int(math.ceil(cw))
+        ch = int(math.ceil(ch))
+
         width = self.width
         height = self.height
 
+        xminimum, yminimum = renpy.display.layout.xyminimums(self.style, width, height)
+
         if not self.style.xfill:
             width = min(cw, width)
 
         if not self.style.yfill:
             height = min(ch, height)
 
-        width = max(width, self.style.xminimum)
-        height = max(height, self.style.yminimum)
+        width = max(width, xminimum)
+        height = max(height, yminimum)
 
-        if self.set_adjustments:
+        if (not renpy.display.render.sizing) and self.set_adjustments:
 
             xarange = max(cw - width, 0)
 
@@ -244,10 +258,19 @@ class Viewport(renpy.display.layout.Container):
 
             self.check_edge_redraw(st)
 
+        redraw = self.xadjustment.periodic(st)
+        if redraw is not None:
+            renpy.display.render.redraw(self, redraw)
+
+        redraw = self.yadjustment.periodic(st)
+        if redraw is not None:
+            renpy.display.render.redraw(self, redraw)
+
         cxo = -int(self.xadjustment.value)
         cyo = -int(self.yadjustment.value)
 
-        self._clipping = (cw > width) or (ch > height)
+        self.width = width
+        self.height = height
 
         return cxo, cyo, width, height
 
@@ -276,7 +299,7 @@ class Viewport(renpy.display.layout.Container):
 
         return rv
 
-    def check_edge_redraw(self, st):
+    def check_edge_redraw(self, st, reset_st=True):
         redraw = False
 
         if (self.edge_xspeed > 0) and (self.xadjustment.value < self.xadjustment.range):
@@ -291,7 +314,8 @@ class Viewport(renpy.display.layout.Container):
 
         if redraw:
             renpy.display.render.redraw(self, 0)
-            self.edge_last_st = st
+            if reset_st or self.edge_last_st is None:
+                self.edge_last_st = st
         else:
             self.edge_last_st = None
 
@@ -300,36 +324,111 @@ class Viewport(renpy.display.layout.Container):
         self.xoffset = None
         self.yoffset = None
 
-        rv = super(Viewport, self).event(ev, x, y, st)
-
-        if rv is not None:
-            return rv
-
-        if self.draggable and renpy.display.focus.get_grab() == self:
-
-            oldx, oldy = self.drag_position
-            dx = x - oldx
-            dy = y - oldy
-
-            self.xadjustment.change(self.xadjustment.value - dx)
-            self.yadjustment.change(self.yadjustment.value - dy)
-
-            self.drag_position = (x, y)  # W0201
-
-            if renpy.display.behavior.map_event(ev, 'viewport_drag_end'):
-                renpy.display.focus.set_grab(None)
-                raise renpy.display.core.IgnoreEvent()
-
         if not ((0 <= x < self.width) and (0 <= y <= self.height)):
             self.edge_xspeed = 0
             self.edge_yspeed = 0
+            self.edge_last_st = None
 
             inside = False
 
         else:
-
             inside = True
 
+        # True if the player can drag the viewoport.
+        draggable = self.draggable and (self.xadjustment.range or self.yadjustment.range)
+
+        grab = renpy.display.focus.get_grab()
+
+        if draggable:
+            if grab is None and renpy.display.behavior.map_event(ev, 'viewport_drag_end'):
+                self.drag_position = None
+        else:
+            self.drag_position = None
+
+        if inside and draggable and (self.drag_position is not None) and (grab is not self):
+
+            focused = renpy.display.focus.get_focused()
+
+            if (focused is None) or (focused is self) or not focused._draggable:
+
+                if ev.type == pygame.MOUSEMOTION:
+
+                    oldx, oldy = self.drag_position
+
+                    if math.hypot(oldx - x, oldy - y) >= renpy.config.viewport_drag_radius:
+                        rv = renpy.display.focus.force_focus(self)
+                        renpy.display.focus.set_grab(self)
+                        self.drag_position = (x, y)
+                        self.drag_position_time = st
+                        self.drag_speed = (0.0, 0.0)
+                        grab = self
+
+                        if rv is not None:
+                            return rv
+
+        if renpy.display.focus.get_grab() == self:
+
+            old_xvalue = self.xadjustment.value
+            old_yvalue = self.yadjustment.value
+
+            oldx, oldy = self.drag_position # type: ignore
+            dx = x - oldx
+            dy = y - oldy
+
+            dt = st - self.drag_position_time
+            if dt > 0:
+                old_xspeed, old_yspeed = self.drag_speed
+                new_xspeed = -dx / dt / 60
+                new_yspeed = -dy / dt / 60
+
+                done = min(1.0, dt / (1 / 60))
+
+                new_xspeed = old_xspeed + done * (new_xspeed - old_xspeed)
+                new_yspeed = old_yspeed + done * (new_yspeed - old_yspeed)
+
+                self.drag_speed = (new_xspeed, new_yspeed)
+
+            if renpy.display.behavior.map_event(ev, 'viewport_drag_end'):
+
+                renpy.display.focus.set_grab(None)
+
+                xspeed, yspeed = self.drag_speed
+
+                if xspeed and renpy.config.viewport_inertia_amplitude:
+                    self.xadjustment.inertia(renpy.config.viewport_inertia_amplitude * xspeed, renpy.config.viewport_inertia_time_constant, st)
+                else:
+                    xvalue = self.xadjustment.round_value(old_xvalue, release=True)
+                    self.xadjustment.change(xvalue)
+
+                if yspeed and renpy.config.viewport_inertia_amplitude:
+                    self.yadjustment.inertia(renpy.config.viewport_inertia_amplitude * yspeed, renpy.config.viewport_inertia_time_constant, st)
+                else:
+                    yvalue = self.yadjustment.round_value(old_yvalue, release=True)
+                    self.yadjustment.change(yvalue)
+
+                self.drag_position = None
+                self.drag_position_time = None
+
+                raise renpy.display.core.IgnoreEvent()
+
+
+            new_xvalue = self.xadjustment.round_value(old_xvalue - dx, release=False)
+            if old_xvalue == new_xvalue:
+                newx = oldx
+            else:
+                self.xadjustment.change(new_xvalue)
+                newx = x
+
+            new_yvalue = self.yadjustment.round_value(old_yvalue - dy, release=False)
+            if old_yvalue == new_yvalue:
+                newy = oldy
+            else:
+                self.yadjustment.change(new_yvalue)
+                newy = y
+
+            self.drag_position = (newx, newy) # W0201
+            self.drag_position_time = st
+
         if inside and self.mousewheel:
 
             if self.mousewheel == "horizontal-change":
@@ -345,7 +444,7 @@ class Viewport(renpy.display.layout.Container):
                 adjustment = self.yadjustment
                 change = False
 
-            if renpy.display.behavior.map_event(ev, 'viewport_up'):
+            if renpy.display.behavior.map_event(ev, 'viewport_wheelup'):
 
                 if change and (adjustment.value == 0):
                     return None
@@ -356,7 +455,7 @@ class Viewport(renpy.display.layout.Container):
                 else:
                     raise renpy.display.core.IgnoreEvent()
 
-            if renpy.display.behavior.map_event(ev, 'viewport_down'):
+            if renpy.display.behavior.map_event(ev, 'viewport_wheeldown'):
 
                 if change and (adjustment.value == adjustment.range):
                     return None
@@ -431,17 +530,18 @@ class Viewport(renpy.display.layout.Container):
                 else:
                     raise renpy.display.core.IgnoreEvent()
 
-        if inside and self.draggable:
+        if inside and draggable:
 
-            if renpy.display.behavior.map_event(ev, 'viewport_drag_start'):
-
-                focused = renpy.display.focus.get_focused()
-
-                if (focused is None) or (focused is self):
+            focused = renpy.display.focus.get_focused()
 
+            if (focused is self) or (focused is None) or (not focused._draggable):
+                if renpy.display.behavior.map_event(ev, 'viewport_drag_start'):
                     self.drag_position = (x, y)
-                    renpy.display.focus.set_grab(self)
-                    raise renpy.display.core.IgnoreEvent()
+                    self.drag_position_time = st
+                    self.drag_speed = (0.0, 0.0)
+
+                    self.xadjustment.end_animation(instantly=True)
+                    self.yadjustment.end_animation(instantly=True)
 
         if inside and self.edge_size and ev.type in [ pygame.MOUSEMOTION, pygame.MOUSEBUTTONDOWN, pygame.MOUSEBUTTONUP ]:
 
@@ -472,10 +572,15 @@ class Viewport(renpy.display.layout.Container):
             self.edge_yspeed = self.edge_speed * self.edge_function(yspeed)
 
             if xspeed or yspeed:
-                self.check_edge_redraw(st)
+                self.check_edge_redraw(st, reset_st=False)
             else:
                 self.edge_last_st = None
 
+        rv = super(Viewport, self).event(ev, x, y, st)
+
+        if rv is not None:
+            return rv
+
         return None
 
     def set_xoffset(self, offset):
@@ -488,14 +593,20 @@ class Viewport(renpy.display.layout.Container):
 
 
 # For compatibility with old saves.
-renpy.display.layout.Viewport = Viewport
+renpy.display.layout.Viewport = Viewport # type: ignore
 
 
 class VPGrid(Viewport):
 
     __version__ = Viewport.__version__
 
-    def __init__(self, cols=None, rows=None, transpose=None, style="vpgrid", **properties):
+    allow_underfull = None
+
+    def __init__(self, cols=None, rows=None,
+                 transpose=None,
+                 style="vpgrid",
+                 allow_underfull=None,
+                 **properties):
 
         super(VPGrid, self).__init__(style=style, **properties)
 
@@ -508,6 +619,7 @@ class VPGrid(Viewport):
         self.grid_cols = cols
         self.grid_rows = rows
         self.grid_transpose = transpose
+        self.allow_underfull = allow_underfull
 
     def render(self, width, height, st, at):
 
@@ -529,7 +641,7 @@ class VPGrid(Viewport):
         rows = self.grid_rows
 
         if cols is None:
-            cols = lc // rows
+            cols = lc // rows # type: ignore
             if rows * cols < lc:
                 cols += 1
 
@@ -547,21 +659,32 @@ class VPGrid(Viewport):
         if yspacing is None:
             yspacing = self.style.spacing
 
+        if renpy.config.relative_spacing:
+            xspacing = renpy.display.layout.scale(xspacing, width)
+            yspacing = renpy.display.layout.scale(yspacing, height)
+
+        left_margin = renpy.display.layout.scale(self.style.left_margin, width)
+        right_margin = renpy.display.layout.scale(self.style.right_margin, width)
+        top_margin = renpy.display.layout.scale(self.style.top_margin, height)
+        bottom_margin = renpy.display.layout.scale(self.style.bottom_margin, height)
+
         rend = renpy.display.render.render(self.children[0], child_width, child_height, st, at)
         cw, ch = rend.get_size()
 
-        tw = (cw + xspacing) * cols - xspacing
-        th = (ch + yspacing) * rows - yspacing
+        tw = (cw + xspacing) * cols - xspacing + left_margin + right_margin
+        th = (ch + yspacing) * rows - yspacing + top_margin + bottom_margin
 
         if self.style.xfill:
             tw = child_width
-            cw = (tw - (cols - 1) * xspacing) / cols
+            cw = (tw - (cols - 1) * xspacing - left_margin - right_margin) // cols
 
         if self.style.yfill:
             th = child_height
-            ch = (th - (rows - 1) * yspacing) / rows
+            ch = (th - (rows - 1) * yspacing - top_margin - bottom_margin) // rows
 
         cxo, cyo, width, height = self.update_offsets(tw, th, st)
+        cxo += left_margin
+        cyo += top_margin
 
         self.offsets = [ ]
 
@@ -607,3 +730,46 @@ class VPGrid(Viewport):
             rv.add_focus(self, None, 0, 0, width, height)
 
         return rv
+
+    def add(self, d):
+        super(VPGrid, self).add(d)
+
+        if None not in (self.grid_cols, self.grid_rows):
+            if len(self.children) > (self.grid_cols * self.grid_rows):
+                if not renpy.config.allow_unfull_vpgrids:
+                    raise Exception("VPGrid overfull.")
+
+    def per_interact(self):
+        super(VPGrid, self).per_interact()
+
+        children = len(self.children)
+
+        given = self.grid_cols or self.grid_rows # ignore if both are 0
+
+        if self.grid_cols is None or self.grid_rows is None:
+            delta = given - (children % given or given) if given else 0
+        else:
+            delta = (self.grid_cols * self.grid_rows) - children
+
+        if not delta:
+            return
+
+        if renpy.config.developer:
+            allow_underfull = self.allow_underfull
+
+            if allow_underfull is None:
+                allow_underfull = renpy.config.allow_underfull_grids or \
+                                  renpy.config.allow_unfull_vpgrids
+
+            if not allow_underfull:
+                msg = "VPGrid not completely full"
+
+                if self.grid_cols is None or self.grid_rows is None:
+                    msg += ", needs a multiple of {} children".format(given)
+
+                raise Exception(msg + ".")
+
+        null = renpy.display.layout.Null()
+
+        for _ in range(delta):
+            self.add(null)
