@@ -1,4 +1,4 @@
-# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2023 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -49,7 +49,7 @@ def file_exists(fn):
     rv = file_exists_cache.get(fn, None)
 
     if rv is None:
-        fullfn = renpy.parser.unelide_filename(fn)
+        fullfn = renpy.lexer.unelide_filename(fn)
 
         rv = os.path.exists(fullfn)
         file_exists_cache[fn] = rv
