@@ -1,4 +1,4 @@
-# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2023 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -25,10 +25,9 @@
 from __future__ import division, absolute_import, with_statement, print_function, unicode_literals
 from renpy.compat import PY2, basestring, bchr, bord, chr, open, pystr, range, round, str, tobytes, unicode # *
 
-
-
 import renpy
 
+import collections
 import hashlib
 import os
 import difflib
@@ -36,6 +35,7 @@ import time
 import marshal
 import struct
 import zlib
+import sys
 
 from renpy.compat.pickle import loads, dumps
 import shutil
@@ -48,6 +48,7 @@ BYTECODE_VERSION = 1
 
 # The python magic code.
 if PY2:
+    import heapq
     import imp
     MAGIC = imp.get_magic()
 
@@ -63,8 +64,10 @@ else:
 # A string at the start of each rpycv2 file.
 RPYC2_HEADER = b"RENPY RPC2"
 
-# A string
-BYTECODE_FILE = "cache/bytecode.rpyb"
+
+# The name of the obsolete and new bytecode cache files.
+OLD_BYTECODE_FILE = "cache/bytecode.rpyb"
+BYTECODE_FILE = "cache/bytecode-{}{}.rpyb".format(sys.version_info.major, sys.version_info.minor)
 
 
 class ScriptError(Exception):
@@ -121,7 +124,8 @@ class Script(object):
         renpy.game.script = self
 
         if os.path.exists(renpy.config.renpy_base + "/lock.txt"):
-            self.key = open(renpy.config.renpy_base + "/lock.txt", "rb").read()
+            with open(renpy.config.renpy_base + "/lock.txt", "rb") as f:
+                self.key = f.read()
         else:
             self.key = None
 
@@ -165,8 +169,7 @@ class Script(object):
             if renpy.loader.loadable(i):
                 return None
 
-        import __main__
-        backups = __main__.path_to_saves(renpy.config.gamedir, "backups") # @UndefinedVariable
+        backups = renpy.__main__.path_to_saves(renpy.config.gamedir, "backups") # @UndefinedVariable
 
         if backups is None:
             return
@@ -223,7 +226,7 @@ class Script(object):
                 continue
 
             try:
-                os.makedirs(os.path.dirname(target_fn), 0o700)
+                os.makedirs(os.path.dirname(target_fn), 0o700) # type: ignore
             except Exception:
                 pass
 
@@ -249,15 +252,24 @@ class Script(object):
 
         for dir, fn in dirlist: # @ReservedAssignment
 
-            if fn.endswith(".rpy"):
+            if fn.endswith("_ren.py"):
+                if dir is None:
+                    continue
+
+                fn = fn[:-7]
+                target = self.script_files
+
+            elif fn.endswith(".rpy"):
                 if dir is None:
                     continue
 
                 fn = fn[:-4]
                 target = self.script_files
+
             elif fn.endswith(".rpyc"):
                 fn = fn[:-5]
                 target = self.script_files
+
             elif fn.endswith(".rpym"):
                 if dir is None:
                     continue
@@ -284,25 +296,22 @@ class Script(object):
 
         initcode = [ ]
 
+        count = 0
+
         for fn, dir in script_files: # @ReservedAssignment
-            # Mitigate "busy script" warning from the browser
-            if renpy.emscripten:
-                import emscripten # type: ignore
-                emscripten.sleep(0)
+
+            count += 1
+            renpy.display.presplash.progress("Loading script...", count, len(script_files))
 
             # Pump the presplash window to prevent marking
             # our process as unresponsive by OS
             renpy.display.presplash.pump_window()
 
-            self.load_appropriate_file(".rpyc", ".rpy", dir, fn, initcode)
+            self.load_appropriate_file(".rpyc", [ "_ren.py", ".rpy" ], dir, fn, initcode)
 
-        # Make the sort stable.
-        initcode = [ (prio, index, code) for index, (prio, code) in
-                     enumerate(initcode) ]
+        initcode.sort(key=lambda i: i[0])
 
-        initcode.sort(key=lambda i: (i[0], i[1]))
-
-        self.initcode = [ (prio, code) for prio, index, code in initcode ]
+        self.initcode = initcode
 
         self.translator.chain_translates()
 
@@ -319,15 +328,42 @@ class Script(object):
         fn, dir = files[0] # @ReservedAssignment
         initcode = [ ]
 
-        self.load_appropriate_file(".rpymc", ".rpym", dir, fn, initcode)
+        self.load_appropriate_file(".rpymc", [ ".rpym" ], dir, fn, initcode)
 
         if renpy.parser.report_parse_errors():
             raise SystemExit(-1)
 
+        initcode.sort(key=lambda i: i[0])
+
         self.translator.chain_translates()
 
         return initcode
 
+    def include_module(self, name):
+        """
+        Loads a module with the provided name and inserts its
+        initcode into the script current initcode
+        """
+        module_initcode = self.load_module(name)
+        if not module_initcode:
+            return
+
+        # We may not insert elements at or prior the current id!
+        current_id = renpy.game.initcode_ast_id
+
+        if module_initcode[0][0] < self.initcode[current_id][0]:
+            raise Exception("Module %s contains nodes with priority lower than the node that loads it" % name)
+
+        merge_id = current_id + 1
+        current_tail = self.initcode[merge_id:]
+
+        # Since script initcode and module initcode are both sorted,
+        # we can use heap to merge them
+        new_tail = current_tail +  module_initcode
+        new_tail.sort(key=lambda i: i[0])
+
+        self.initcode[merge_id:] = new_tail
+
     def assign_names(self, stmts, fn):
         # Assign names to statements that don't have one already.
 
@@ -385,6 +421,8 @@ class Script(object):
 
         stmts = self.finish_load(stmts, initcode, False)
 
+        initcode.sort(key=lambda i: i[0])
+
         return stmts, initcode
 
     def finish_load(self, stmts, initcode, check_names=True, filename=None):
@@ -428,7 +466,7 @@ class Script(object):
 
         # Fix the filename for a renamed .rpyc file.
         if filename is not None:
-            filename = renpy.parser.elide_filename(filename)
+            filename = renpy.lexer.elide_filename(filename)
 
             if not all_stmts[0].filename.lower().endswith(filename.lower()):
 
@@ -473,9 +511,9 @@ class Script(object):
                     self.duplicate_labels.append(
                         u'The label {} is defined twice, at File "{}", line {}:\n{}and File "{}", line {}:\n{}'.format(
                             bad_name, old_node.filename, old_node.linenumber,
-                            renpy.parser.get_line_text(old_node.filename, old_node.linenumber),
+                            renpy.lexer.get_line_text(old_node.filename, old_node.linenumber),
                             bad_node.filename, bad_node.linenumber,
-                            renpy.parser.get_line_text(bad_node.filename, bad_node.linenumber),
+                            renpy.lexer.get_line_text(bad_node.filename, bad_node.linenumber),
                         ))
 
         self.update_bytecode()
@@ -592,122 +630,149 @@ class Script(object):
 
     def load_file(self, dir, fn): # @ReservedAssignment
 
-        if fn.endswith(".rpy") or fn.endswith(".rpym"):
+        # Used to only find the deferred parse errors from this file.
+        old_deferred_parse_errors = renpy.parser.deferred_parse_errors
+        renpy.parser.deferred_parse_errors = collections.defaultdict(list)
 
-            if not dir:
-                raise Exception("Cannot load rpy/rpym file %s from inside an archive." % fn)
+        try:
 
-            base, _, game = dir.rpartition("/")
-            olddir = base + "/old-" + game
+            if fn.endswith(".rpy") or fn.endswith(".rpym") or fn.endswith("_ren.py"):
 
-            fullfn = dir + "/" + fn
-            rpycfn = fullfn + "c"
+                if not dir:
+                    raise Exception("Cannot load rpy/rpym/ren.py file %s from inside an archive." % fn)
 
-            oldrpycfn = olddir + "/" + fn + "c"
+                base, _, game = dir.rpartition("/")
+                olddir = base + "/old-" + game
 
-            stmts = renpy.parser.parse(fullfn)
+                fullfn = dir + "/" + fn
 
-            data = { }
-            data['version'] = script_version
-            data['key'] = self.key or 'unlocked'
+                if fn.endswith("_ren.py"):
+                    rpycfn = fullfn[:-7] + ".rpyc"
+                    oldrpycfn = olddir + "/" + fn[:-7] + ".rpyc"
+                else:
+                    rpycfn = fullfn + "c"
+                    oldrpycfn = olddir + "/" + fn + "c"
 
-            if stmts is None:
-                return data, [ ]
+                stmts = renpy.parser.parse(fullfn)
 
-            used_names = set()
+                data = { }
+                data['version'] = script_version
+                data['key'] = self.key or 'unlocked'
+                data['deferred_parse_errors'] = renpy.parser.deferred_parse_errors
 
-            for mergefn in [ oldrpycfn, rpycfn ]:
+                if stmts is None:
+                    return data, [ ]
 
-                # See if we have a corresponding .rpyc file. If so, then
-                # we want to try to upgrade our .rpy file with it.
-                try:
-                    self.record_pycode = False
+                used_names = set()
 
-                    with open(mergefn, "rb") as rpycf:
-                        bindata = self.read_rpyc_data(rpycf, 1)
+                for mergefn in [ oldrpycfn, rpycfn ]:
 
-                    if bindata is not None:
-                        old_data, old_stmts = loads(bindata)
-                        self.merge_names(old_stmts, stmts, used_names)
-
-                    del old_data
-                    del old_stmts
-                except Exception:
-                    pass
-                finally:
-                    self.record_pycode = True
-
-            self.assign_names(stmts, renpy.parser.elide_filename(fullfn))
-
-            pickle_data_before_static_transforms = dumps((data, stmts))
-
-            self.static_transforms(stmts)
-
-            pickle_data_after_static_transforms = dumps((data, stmts))
-
-            if not renpy.macapp:
-                try:
-                    with open(rpycfn, "wb") as f:
-                        self.write_rpyc_header(f)
-                        self.write_rpyc_data(f, 1, pickle_data_before_static_transforms)
-                        self.write_rpyc_data(f, 2, pickle_data_after_static_transforms)
-
-                        with open(fullfn, "rb") as fullf:
-                            rpydigest = hashlib.md5(fullf.read()).digest()
-
-                        self.write_rpyc_md5(f, rpydigest)
-                except Exception:
-                    import traceback
-                    traceback.print_exc()
-
-            self.loaded_rpy = True
-
-        elif fn.endswith(".rpyc") or fn.endswith(".rpymc"):
-
-            data = None
-            stmts = None
-
-            with renpy.loader.load(fn) as f:
-                for slot in [ 2, 1 ]:
+                    # See if we have a corresponding .rpyc file. If so, then
+                    # we want to try to upgrade our .rpy file with it.
                     try:
-                        bindata = self.read_rpyc_data(f, slot)
+                        self.record_pycode = False
 
-                        if bindata:
-                            data, stmts = loads(bindata)
-                            break
+                        with open(mergefn, "rb") as rpycf:
+                            bindata = self.read_rpyc_data(rpycf, 1)
 
+                        if bindata is not None:
+                            old_data, old_stmts = loads(bindata)
+                            self.merge_names(old_stmts, stmts, used_names)
+
+                            del old_data
+                            del old_stmts
                     except Exception:
                         pass
+                    finally:
+                        self.record_pycode = True
 
-                    f.seek(0)
+                self.assign_names(stmts, renpy.lexer.elide_filename(fullfn))
 
-                else:
-                    return None, None
+                pickle_data_before_static_transforms = dumps((data, stmts))
 
-                if data is None:
-                    print("Failed to load", fn)
-                    return None, None
+                self.static_transforms(stmts)
 
-                if not isinstance(data, dict):
-                    return None, None
+                pickle_data_after_static_transforms = dumps((data, stmts))
 
-                if self.key and data.get('key', 'unlocked') != self.key:
-                    return None, None
+                if not renpy.macapp:
+                    try:
+                        with open(rpycfn, "wb") as f:
+                            self.write_rpyc_header(f)
+                            self.write_rpyc_data(f, 1, pickle_data_before_static_transforms)
+                            self.write_rpyc_data(f, 2, pickle_data_after_static_transforms)
 
-                if data['version'] != script_version:
-                    return None, None
+                            with open(fullfn, "rb") as fullf:
+                                rpydigest = hashlib.md5(fullf.read()).digest()
 
-                if slot < 2:
-                    self.static_transforms(stmts)
+                            self.write_rpyc_md5(f, rpydigest)
+                    except Exception:
+                        import traceback
+                        traceback.print_exc()
 
-        else:
-            return None, None
+                self.loaded_rpy = True
 
-        return data, stmts
+            elif fn.endswith(".rpyc") or fn.endswith(".rpymc"):
 
-    def load_appropriate_file(self, compiled, source, dir, fn, initcode): # @ReservedAssignment
+                data = None
+                stmts = None
+
+                with renpy.loader.load(fn, tl=False) as f:
+                    for slot in [ 2, 1 ]:
+                        try:
+                            bindata = self.read_rpyc_data(f, slot)
+
+                            if bindata:
+                                data, stmts = loads(bindata)
+                                break
+
+                        except Exception:
+                            pass
+
+                        f.seek(0)
+
+                    else:
+                        return None, None
+
+                    if data is None:
+                        print("Failed to load", fn)
+                        return None, None
+
+                    if not isinstance(data, dict):
+                        return None, None
+
+                    if self.key and data.get('key', 'unlocked') != self.key:
+                        return None, None
+
+                    if data['version'] != script_version:
+                        return None, None
+
+                    if slot < 2:
+                        self.static_transforms(stmts)
+
+                    renpy.parser.deferred_parse_errors = data.get('deferred_parse_errors', None) or collections.defaultdict(list)
+
+            else:
+                return None, None
+
+            return data, stmts
+
+        finally:
+
+            # Restore the deferred parse errors.
+            for k, v in renpy.parser.deferred_parse_errors.items():
+                old_deferred_parse_errors[k].extend(v)
+
+            renpy.parser.deferred_parse_errors = old_deferred_parse_errors
+
+
+
+    def load_appropriate_file(self, compiled, source_extensions, dir, fn, initcode): # @ReservedAssignment
         data = None
 
+        source = source_extensions[-1]
+
+        renpy.game.exception_info = "While loading the script."
+
         # This can only be a .rpyc file, since we're loading it
         # from an archive.
         if dir is None:
@@ -719,7 +784,7 @@ class Script(object):
             if data is None:
                 raise Exception("Could not load from archive %s." % (lastfn,))
 
-            with renpy.loader.load(fn + compiled) as f:
+            with renpy.loader.load(fn + compiled, tl=False) as f:
                 f.seek(-hashlib.md5().digest_size, 2)
                 digest = f.read(hashlib.md5().digest_size)
 
@@ -727,16 +792,30 @@ class Script(object):
 
             # Otherwise, we're loading from disk. So we need to decide if
             # we want to load the rpy or the rpyc file.
-            rpyfn = dir + "/" + fn + source
             rpycfn = dir + "/" + fn + compiled
+            rpyfn = None # prevent the spurious warning.
+            rpydigest = None
 
-            renpy.loader.add_auto(rpyfn)
+            rpyfns = [ ]
+
+            for source in source_extensions:
+                rpyfn = dir + "/" + fn + source
+
+                renpy.loader.add_auto(rpyfn)
+
+                if os.path.exists(rpyfn):
+                    rpyfns.append((source, rpyfn))
+
+            if len(rpyfns) > 1:
+                raise Exception("{} conflict, and can't exist in the same game.".format(" and ".join(i[1] for i in rpyfns)))
+            elif rpyfns:
+                source, rpyfn = rpyfns[0]
 
-            if os.path.exists(rpyfn):
                 with open(rpyfn, "rb") as f:
                     rpydigest = hashlib.md5(f.read()).digest()
             else:
-                rpydigest = None
+                source = source_extensions[-1]
+                rpyfn = dir + "/" + fn + source_extensions[-1]
 
             try:
                 if os.path.exists(rpycfn):
@@ -841,6 +920,7 @@ class Script(object):
 
         # Update all of the PyCode objects in the system with the loaded
         # bytecode.
+
         for i in self.all_pycode:
 
             key = i.get_hash() + MAGIC
@@ -905,6 +985,12 @@ class Script(object):
             except Exception:
                 pass
 
+            fn = renpy.loader.get_path(OLD_BYTECODE_FILE)
+            try:
+                os.unlink(fn)
+            except Exception:
+                pass
+
     def lookup(self, label):
         """
         Looks up the given label in the game. If the label is not found,
