@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2023 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -80,9 +80,8 @@ def compile_expr(loc, node):
         flags = renpy.python.new_compile_flags
 
     expr = ast.Expression(body=node)
-    ast.fix_missing_locations(expr)
-    return compile(expr, filename, "eval", flags, 1)
-
+    renpy.python.fix_locations(expr, 1, 0)
+    return compile(expr, filename, "eval", flags, True)
 
 class SLContext(renpy.ui.Addable):
     """
@@ -343,6 +342,29 @@ class SLNode(object):
         profile_log.write("%s", "    {}{}{} ({}:{})".format(const_type, prefix, formatted, self.location[0], self.location[1]))
 
 
+def analyze_keywords(node, analysis, conditional=GLOBAL_CONST):
+    """
+    Analyzes the keywords that can be applied to this statement,
+    including those provided by if statements.
+    """
+
+    rv = GLOBAL_CONST
+
+    for _, expr in node.keyword:
+        rv = min(rv, analysis.is_constant_expr(expr), conditional)
+
+    for n in node.children:
+        if isinstance(n, SLIf):
+
+            for cond, block in n.entries:
+                if cond is not None:
+                    conditional = min(conditional, analysis.is_constant_expr(cond))
+
+                rv = min(rv, analyze_keywords(block, analysis, conditional))
+
+    return rv
+
+
 # A sentinel used to indicate a keyword argument was not given.
 NotGiven = renpy.object.Sentinel("NotGiven")
 
@@ -356,6 +378,9 @@ class SLBlock(SLNode):
     # RawBlock from parse or None if not present.
     atl_transform = None
 
+    # The actual transform created from the atl transform.
+    transform = None
+
     def __init__(self, loc):
         SLNode.__init__(self, loc)
 
@@ -440,6 +465,9 @@ class SLBlock(SLNode):
             const = self.atl_transform.constant
             self.constant = min(self.constant, const)
 
+            self.transform = renpy.display.transform.ATLTransform(self.atl_transform)
+            renpy.atl.compile_queue.append(self.transform)
+
         was_last_keyword = False
         for i in self.children:
             if i.has_keyword:
@@ -486,7 +514,18 @@ class SLBlock(SLNode):
 
         if self.atl_transform is not None:
             transform = ATLTransform(self.atl_transform, context=context.scope)
-            context.keywords["at"] = transform
+            transform.parent_transform = self.transform # type: ignore
+
+            if "at" in context.keywords:
+                try:
+                    at_list = list(context.keywords["at"])
+                except TypeError:
+                    at_list = [ context.keywords["at"] ]
+
+                at_list.append(transform)
+                context.keywords["at"] = at_list
+            else:
+                context.keywords["at"] = transform
 
         style_prefix = context.keywords.pop("style_prefix", NotGiven)
 
@@ -723,12 +762,7 @@ class SLDisplayable(SLBlock):
     def analyze(self, analysis):
 
         if self.imagemap:
-
-            const = GLOBAL_CONST
-
-            for _k, expr in self.keyword:
-                const = min(const, analysis.is_constant_expr(expr))
-
+            const = analyze_keywords(self, analysis)
             analysis.push_control(imagemap=(const != GLOBAL_CONST))
 
         if self.hotspot:
@@ -750,7 +784,7 @@ class SLDisplayable(SLBlock):
             for i in self.positional:
                 const = min(self.constant, analysis.is_constant_expr(i))
 
-            for k, v in self.keyword:
+            for _k, v in self.keyword:
                 const = min(self.constant, analysis.is_constant_expr(v))
 
             if self.keyword_exist("id"):
@@ -924,6 +958,16 @@ class SLDisplayable(SLBlock):
             # Get the widget id and transform, if any.
             widget_id = keywords.pop("id", None)
             transform = keywords.pop("at", None)
+            prefer_screen_to_id = keywords.pop("prefer_screen_to_id", False)
+
+            if widget_id and (widget_id in screen.widget_properties):
+
+                if prefer_screen_to_id:
+                    new_keywords = screen.widget_properties[widget_id].copy()
+                    new_keywords.update(keywords)
+                    keywords = new_keywords
+                else:
+                    keywords.update(screen.widget_properties[widget_id])
 
             # If we don't know the style, figure it out.
             style_suffix = keywords.pop("style_suffix", None) or self.style
@@ -933,9 +977,6 @@ class SLDisplayable(SLBlock):
                 else:
                     keywords["style"] = ctx.style_prefix + "_" + style_suffix
 
-            if widget_id and (widget_id in screen.widget_properties):
-                keywords.update(screen.widget_properties[widget_id])
-
             old_d = cache.displayable
             if old_d:
                 old_main = old_d._main or old_d
@@ -1131,7 +1172,8 @@ class SLDisplayable(SLBlock):
         cache.children = ctx.children
         cache.style_prefix = context.style_prefix
 
-        transform = transform # type: ignore
+        if not transform:
+            transform = None
 
         if (transform is not None) and (d is not NO_DISPLAYABLE):
             if reused and (transform == cache.raw_transform):
@@ -1367,13 +1409,16 @@ class SLIf(SLNode):
             if cond is not None:
                 node = ccache.ast_eval(cond)
 
-                self.constant = min(self.constant, analysis.is_constant(node))
+                cond_const = analysis.is_constant(node)
+                self.constant = min(self.constant, cond_const)
 
                 cond = compile_expr(self.location, node)
+            else:
+                cond_const = True
 
             block.prepare(analysis)
             self.constant = min(self.constant, block.constant)
-            self.prepared_entries.append((cond, block))
+            self.prepared_entries.append((cond, block, cond_const))
 
             self.has_keyword |= block.has_keyword
             self.last_keyword |= block.last_keyword
@@ -1384,7 +1429,7 @@ class SLIf(SLNode):
             self.execute_predicting(context)
             return
 
-        for cond, block in self.prepared_entries:
+        for cond, block, _cond_const in self.prepared_entries:
             if cond is None or eval(cond, context.globals, context.scope):
                 for i in block.children:
                     i.execute(context)
@@ -1397,10 +1442,13 @@ class SLIf(SLNode):
         # True if no block has been the main choice yet.
         first = True
 
-        # Other blocks that we predict if not predicted.
-        false_blocks = [ ]
 
-        for cond, block in self.prepared_entries:
+        # Should we predict false branches?
+        predict_false = self.serial not in context.predicted
+        context.predicted.add(self.serial)
+
+
+        for cond, block, const_cond in self.prepared_entries:
             try:
                 cond_value = (cond is None) or eval(cond, context.globals, context.scope)
             except Exception:
@@ -1416,34 +1464,29 @@ class SLIf(SLNode):
                     except Exception:
                         pass
 
-            else:
-                false_blocks.append(block)
+                if const_cond:
+                    break
 
-        # Has any instance of this node been predicted? We only predict
-        # once per node, for performance reasons.
-        if self.serial in context.predicted:
-            return
+            elif predict_false:
 
-        context.predicted.add(self.serial)
+                ctx = SLContext(context)
+                ctx.children = [ ]
+                ctx.unlikely = True
 
-        # Not-taken branches.
-        for block in false_blocks:
-            ctx = SLContext(context)
-            ctx.children = [ ]
-            ctx.unlikely = True
+                for i in block.children:
+                    try:
+                        i.execute(ctx)
+                    except Exception:
+                        pass
+
+                for i in ctx.children:
+                    predict_displayable(i)
 
-            for i in block.children:
-                try:
-                    i.execute(ctx)
-                except Exception:
-                    pass
 
-            for i in ctx.children:
-                predict_displayable(i)
 
     def keywords(self, context):
 
-        for cond, block in self.prepared_entries:
+        for cond, block, _cond_const in self.prepared_entries:
             if cond is None or eval(cond, context.globals, context.scope):
                 block.keywords(context)
                 return
@@ -1618,16 +1661,27 @@ class SLFor(SLBlock):
 
     def analyze(self, analysis):
 
-        if analysis.is_constant_expr(self.expression) == GLOBAL_CONST:
-            analysis.push_control(True)
-            analysis.mark_constant(self.variable)
-        else:
-            analysis.push_control(False)
-            analysis.mark_not_constant(self.variable)
+        const = analysis.is_constant_expr(self.expression) == GLOBAL_CONST
 
-        SLBlock.analyze(self, analysis)
+        while True:
 
-        analysis.pop_control()
+            if const:
+                analysis.push_control(True, loop=True)
+                analysis.mark_constant(self.variable)
+            else:
+                analysis.push_control(False, loop=True)
+                analysis.mark_not_constant(self.variable)
+
+            SLBlock.analyze(self, analysis)
+
+            new_const = analysis.control.const
+
+            analysis.pop_control()
+
+            if new_const == const:
+                break
+
+            const = new_const
 
     def prepare(self, analysis):
         node = ccache.ast_eval(self.expression)
@@ -1635,7 +1689,7 @@ class SLFor(SLBlock):
         const = analysis.is_constant(node)
 
         if const == GLOBAL_CONST:
-            self.expression_value = py_eval_bytecode(compile_expr(self.location, node))
+            self.expression_value = list(py_eval_bytecode(compile_expr(self.location, node)))
             self.expression_expr = None
         else:
             self.expression_value = None
@@ -1712,12 +1766,19 @@ class SLFor(SLBlock):
 
             # Inline of SLBlock.execute.
 
-            for i in children_i:
-                try:
-                    i.execute(ctx)
-                except Exception:
-                    if not context.predicting:
+            try:
+                for i in children_i:
+                    try:
+                        i.execute(ctx)
+                    except SLForException:
                         raise
+                    except Exception:
+                        if not context.predicting:
+                            raise
+            except SLBreakException:
+                break
+            except SLContinueException:
+                continue
 
             if context.unlikely:
                 break
@@ -1747,6 +1808,44 @@ class SLFor(SLBlock):
         for i in self.children:
             i.dump_const(prefix + "  ")
 
+class SLForException(Exception): pass
+
+class SLBreakException(SLForException): pass
+
+class SLContinueException(SLForException): pass
+
+class SLBreak(SLNode):
+
+    def analyze(self, analysis):
+        analysis.exit_loop()
+
+    def execute(self, context):
+        raise SLBreakException()
+
+    def copy(self, transclude):
+        rv = self.instantiate(transclude)
+
+        return rv
+
+    def dump_const(self, prefix):
+        self.dc(prefix, "break")
+
+class SLContinue(SLNode):
+
+    def analyze(self, analysis):
+        analysis.exit_loop()
+
+    def execute(self, context):
+        raise SLContinueException()
+
+    def copy(self, transclude):
+        rv = self.instantiate(transclude)
+
+        return rv
+
+    def dump_const(self, prefix):
+        self.dc(prefix, "continue")
+
 class SLPython(SLNode):
 
     def __init__(self, loc, code):
@@ -1985,15 +2084,13 @@ class SLUse(SLNode):
 
             ctx.old_cache = context.old_use_cache.get(use_id, None) or context.old_cache.get(self.serial, None) or { }
 
-            if use_id in ctx.old_use_cache:
-                ctx.updating = True
-
             ctx.new_use_cache[use_id] = ctx.new_cache
 
         else:
 
             ctx.old_cache = context.old_cache.get(self.serial, None) or { }
 
+
         if not isinstance(ctx.old_cache, dict):
             ctx.old_cache = { }
         if not isinstance(ctx.miss_cache, dict):
@@ -2013,11 +2110,14 @@ class SLUse(SLNode):
             args = [ ]
             kwargs = { }
 
+        scope = ctx.old_cache.get("scope", None) or ctx.miss_cache.get("scope", None) or { }
+        if not ctx.updating:
+            scope.clear()
+
         # Apply the arguments to the parameters (if present) or to the scope of the used screen.
         if ast.parameters is not None:
             new_scope = ast.parameters.apply(args, kwargs, ignore_errors=context.predicting)
 
-            scope = ctx.old_cache.get("scope", None) or ctx.miss_cache.get("scope", None) or { }
             scope.update(new_scope)
 
         else:
@@ -2025,7 +2125,6 @@ class SLUse(SLNode):
             if args:
                 raise Exception("Screen {} does not take positional arguments. ({} given)".format(self.target, len(args)))
 
-            scope = ctx.old_cache.get("scope", None) or ctx.miss_cache.get("scope", None) or { }
             scope.clear()
             scope.update(context.scope)
             scope.update(kwargs)
@@ -2269,9 +2368,6 @@ class SLCustomUse(SLNode):
 
             ctx.old_cache = context.old_use_cache.get(use_id, None) or context.old_cache.get(self.serial, None) or { }
 
-            if use_id in ctx.old_use_cache:
-                ctx.updating = True
-
             ctx.new_use_cache[use_id] = ctx.new_cache
 
         else:
@@ -2285,11 +2381,14 @@ class SLCustomUse(SLNode):
 
         ast = self.ast
 
+        scope = ctx.old_cache.get("scope", None) or ctx.miss_cache.get("scope", None) or { }
+        if not ctx.updating:
+            scope.clear()
+
         # Apply the arguments to the parameters (if present) or to the scope of the used screen.
         if ast.parameters is not None:
             new_scope = ast.parameters.apply(args, kwargs, ignore_errors=context.predicting)
 
-            scope = ctx.old_cache.get("scope", None) or ctx.miss_cache.get("scope", None) or { }
             scope.update(new_scope)
 
         else:
@@ -2297,7 +2396,6 @@ class SLCustomUse(SLNode):
             if args:
                 raise Exception("Screen {} does not take positional arguments. ({} given)".format(self.target, len(args)))
 
-            scope = ctx.old_cache.get("scope", None) or ctx.miss_cache.get("scope", None) or { }
             scope.clear()
             scope.update(context.scope)
             scope.update(kwargs)
