@@ -1,4 +1,4 @@
-# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2023 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -21,7 +21,7 @@
 
 from __future__ import division, absolute_import, with_statement, print_function, unicode_literals
 from renpy.compat import PY2, basestring, bchr, bord, chr, open, pystr, range, round, str, tobytes, unicode # *
-
+from typing import Any, Callable, Literal
 
 
 import collections
@@ -51,8 +51,11 @@ STYLE_PREFIXES = [
 # The parser that things are being added to.
 parser = None
 
-# All statements we know about.
-all_statements = [ ]
+# The names of all statements known to SL.
+statement_names = set()
+
+# A list of statements that are valid anywhere a child can be placed.
+all_child_statements = [ ]
 
 # Statements that can contain children.
 childbearing_statements = set()
@@ -117,6 +120,21 @@ class PrefixStyle(object):
             parser.add(self)
 
 
+from renpy.styledata.stylesets import proxy_properties as incompatible_props
+
+def check_incompatible_props(new, olds):
+    """
+    Takes a property and a set of already-seen properties, and checks
+    to see if the new is incompatible with any of the old ones.
+    """
+    newly_set = incompatible_props.get(new, set()) | {new}
+
+    for old in olds:
+        if newly_set.intersection(incompatible_props.get(old, (old,))):
+            return old
+
+    return False
+
 class Parser(object):
 
     # The number of children this statement takes, out of 0, 1, or "many".
@@ -124,7 +142,7 @@ class Parser(object):
     # inside something that takes a single child.
     nchildren = "many"
 
-    def __init__(self, name, statement=True):
+    def __init__(self, name, child_statement=True):
 
         # The name of this object.
         self.name = name
@@ -135,11 +153,13 @@ class Parser(object):
         self.keyword = { }
         self.children = { }
 
+        statement_names.add(name)
+
         # True if this parser takes "as".
         self.variable = False
 
-        if statement:
-            all_statements.append(self)
+        if child_statement:
+            all_child_statements.append(self)
 
         global parser
         parser = self
@@ -240,7 +260,7 @@ class Parser(object):
 
             if can_tag and name == "tag":
                 if target.tag is not None:
-                    l.error('keyword argument %r appears more than once in a %s statement.' % (name, self.name))
+                    l.error('the tag keyword argument appears more than once in a %s statement.' % (self.name,))
 
                 target.tag = l.require(l.word)
                 l.expect_noblock(name)
@@ -255,14 +275,18 @@ class Parser(object):
                     return
 
             if name not in self.keyword:
-                l.error('%r is not a keyword argument or valid child for the %s statement.' % (name, self.name))
+                if name == "continue" or name == "break":
+                    l.error("The %s statement may only appear inside a for statement, or an if statement inside a for statement." % name)
+                elif name in statement_names:
+                    l.error('The %s statement is not a valid child of the %s statement.' % (name, self.name))
+                else:
+                    l.error('%r is not a keyword argument or valid child of the %s statement.' % (name, self.name))
 
-            if name in seen_keywords:
-                l.error('keyword argument %r appears more than once in a %s statement.' % (name, self.name))
+            if name == "at" and l.keyword("transform"):
 
-            seen_keywords.add(name)
+                if target.atl_transform is not None:
+                    l.error("More than one 'at transform' block is given.")
 
-            if name == "at" and block and l.keyword("transform"):
                 l.require(":")
                 l.expect_eol()
                 l.expect_block("ATL block")
@@ -270,7 +294,20 @@ class Parser(object):
                 target.atl_transform = expr
                 return
 
+            if name in seen_keywords:
+                l.error('keyword argument %r appears more than once in a %s statement.' % (name, self.name))
+            incomprop = check_incompatible_props(name, seen_keywords)
+            if incomprop:
+                l.deferred_error("check_conflicting_properties", 'keyword argument {!r} is incompatible with {!r}.'.format(name, incomprop))
+
+            if name == "at" and target.atl_transform:
+                l.error("The 'at' property must occur before the 'at transform' block.")
+
+            seen_keywords.add(name)
+
             expr = l.comma_expression()
+            if expr is None:
+                l.error("the {} keyword argument was not given a value.".format(name))
 
             if (not keyword) and (not renpy.config.keyword_after_python):
                 try:
@@ -300,7 +337,8 @@ class Parser(object):
                     break
 
                 if l.eol():
-                    l.expect_noblock(self.name)
+                    if not target.atl_transform:
+                        l.expect_noblock(self.name)
                     block = False
                     break
 
@@ -422,7 +460,7 @@ many = renpy.object.Sentinel("many")
 def register_sl_displayable(*args, **kwargs):
     """
     :doc: custom_sl class
-    :args: (name, displayable, style, nchildren=0, scope=False, replaces=False, default_keywords={}, default_properties=True, unique=False)
+    :args: (name, displayable, style, nchildren=0, scope=False, *, replaces=False, default_keywords={}, default_properties=True, unique=False)
 
     Registers a screen language statement that creates a displayable.
 
@@ -514,7 +552,7 @@ def register_sl_displayable(*args, **kwargs):
         * "text"
         * "window"
 
-        These correspond to groups of :ref:`style-properties`. Group can
+        These correspond to groups of :doc:`style_properties`. Group can
         also be "ui", in which case it adds the :ref:`common ui properties <common-properties>`.
     """
 
@@ -530,7 +568,7 @@ def register_sl_displayable(*args, **kwargs):
     if rv.nchildren != 0:
         childbearing_statements.add(rv)
 
-        for i in all_statements:
+        for i in all_child_statements:
             rv.add(i)
 
     rv.add(if_statement)
@@ -543,10 +581,10 @@ class DisplayableParser(Parser):
 
     def __init__(self, name, displayable, style, nchildren=0, scope=False,
                  pass_context=False, imagemap=False, replaces=False, default_keywords={},
-                 hotspot=False, default_properties=True, unique=False):
+                 hotspot=False, default_properties=True, unique=False): # type: (str, Callable, str|None, int|Literal["many"]|renpy.object.Sentinel, bool, bool, bool, bool, dict[str, Any], bool, bool, bool) -> None
         """
         `scope`
-            If true, the scope is passed into the displayable functionas a keyword
+            If true, the scope is passed into the displayable function as a keyword
             argument named "scope".
 
         `pass_context`
@@ -591,6 +629,7 @@ class DisplayableParser(Parser):
 
         Keyword("arguments")
         Keyword("properties")
+        Keyword("prefer_screen_to_id")
 
         if default_properties:
             add(renpy.sl2.slproperties.ui_properties)
@@ -792,7 +831,30 @@ class ForParser(Parser):
         return rv
 
 
-ForParser("for")
+for_parser = ForParser("for")
+
+
+class BreakParser(Parser):
+
+    def parse(self, loc, l, parent, keyword):
+
+        l.expect_eol()
+        l.expect_noblock('break statement')
+
+        return slast.SLBreak(loc)
+
+for_parser.add(BreakParser("break", False))
+
+class ContinueParser(Parser):
+
+    def parse(self, loc, l, parent, keyword):
+
+        l.expect_eol()
+        l.expect_noblock('continue statement')
+
+        return slast.SLContinue(loc)
+
+for_parser.add(ContinueParser("continue", False))
 
 
 class OneLinePythonParser(Parser):
@@ -953,7 +1015,7 @@ class CustomParser(Parser):
         if self.nchildren != 0:
             childbearing_statements.add(self)
 
-            for i in all_statements:
+            for i in all_child_statements:
                 self.add(i)
 
         self.add_property("arguments")
@@ -999,7 +1061,7 @@ class CustomParser(Parser):
 class ScreenParser(Parser):
 
     def __init__(self):
-        super(ScreenParser, self).__init__("screen", statement=False)
+        super(ScreenParser, self).__init__("screen", child_statement=False)
 
     def parse(self, loc, l, parent, name="_name", keyword=True):
 
@@ -1037,12 +1099,12 @@ parser = None
 
 
 def init():
-    screen_parser.add(all_statements)
+    screen_parser.add(all_child_statements)
 
-    for i in all_statements:
+    for i in all_child_statements:
 
         if i in childbearing_statements:
-            i.add(all_statements)
+            i.add(all_child_statements)
         else:
             i.add(if_statement)
             i.add(pass_statement)
