@@ -1,4 +1,4 @@
-# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2023 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -56,6 +56,8 @@ def register(
         post_label=None,
         predict_all=True,
         predict_next=None,
+        execute_default=None,
+        reachable=None,
 ):
     """
     :doc: statement_register
@@ -89,7 +91,8 @@ def register(
         single argument, the object returned from parse.
 
     `execute_init`
-        This is a function that is called at init time, at priority 0.
+        This is a function that is called at init time, at priority 0. It is passed a
+        single argument, the object returned from parse.
 
     `predict`
         This is a function that is called to predict the images used by the statement.
@@ -105,7 +108,9 @@ def register(
 
         The function should return either a string giving a label to jump to,
         the second argument to transfer control into the block, or None to
-        continue to the statement after this one.
+        continue to the statement after this one. It can also return the result
+        of :meth:`Lexer.renpy_statement` or :meth:`Lexer.renpy_block` when
+        called in the `parse` function.
 
     `label`
         This is a function that is called to determine the label of this
@@ -124,12 +129,15 @@ def register(
     `init`
         True if this statement should be run at init-time. (If the statement
         is not already inside an init block, it's automatically placed inside
-        an init block.) This calls the execute function, in addition to the
-        execute_init function.
+        an init block.)
+
+        You probably don't want this if you have an `execute_init` function,
+        as wrapping the statement in an init block will cause the `execute_init`
+        and `execute` functions to be called at the same time.
 
     `init_priority`
         An integer that determines the priority of initialization of the
-        init block.
+        init block created by `init` and `execute_init` function.
 
     `translation_strings`
         A function that is called with the parsed block. It's expected to
@@ -162,6 +170,57 @@ def register(
         This should be called to predict the statements that can run after
         this one. It's expected to return a list of of labels or SubParse
         objects. This is not called if `predict_all` is true.
+
+    `execute_default`
+        This is a function that is called at the same time the default
+        statements are run - after the init phase, but before the game starts; when the
+        a save is loaded; after rollback; before lint; and potentially at
+        other times.
+
+        This is called with a single argument, the object returned from parse.
+
+    `reachable`
+        This is a function that is called to allow this statement to
+        customize how it participates in lint's reachability analysis.
+
+        By default, a statement's custom block, sub-parse blocks created
+        with Lexer.renpy_block(), and the statement after the statement
+        are reachable if the statement itself is reachable. The statement
+        is also reachable if it has a label function.
+
+        This can be customized by providing a reachable function. This is
+        a function that takes five arguments (in the following, a "label"
+        may be a string or an opaque object):
+
+        * The object returned by the parse function.
+        * A boolean that is true if the statement is reachable.
+        * The label of the statement.
+        * The label of the next statement, or None if there is no next statement.
+        * If `block` is set to "script", the label of the first statement in the block,
+          or None if there is no block.
+
+        It's expected to return a set that may contain:
+
+        * A label or subparse object of a statement that is reachable.
+        * True, to indicate that this statement should not be reported by lint,
+          but is not intrinsically reachable. (It will become reachable if it
+          is reported reachable by another statement.)
+        * None, which is ignored.
+
+        This function may be called multiple times with both value of is_reachable,
+        to allow the statement to customize its behavior based on whether it's
+        reachable or not. (For example, the next statement may only be reachable
+        if this statement is.)
+
+    .. warning::
+
+        Using the empty string as the name to redefine the say statement is
+        usually a bad idea. That is because when replacing a Ren'Py native
+        statement, its behavior depends on the :doc:`statement_equivalents`. In
+        the case of the say statement, these equivalents do not support the `id`
+        and translation systems. As a result, a game redefining the default
+        statement will not be able to use these features (short of
+        reimplementing them entirely).
     """
 
     name = tuple(name.split())
@@ -185,7 +244,8 @@ def register(
         post_label=post_label,
         predict_all=predict_all,
         predict_next=predict_next,
-
+        execute_default=execute_default,
+        reachable=reachable,
     )
 
     if block not in [True, False, "script", "possible" ]:
@@ -226,6 +286,7 @@ def register(
             rv.translation_relevant = bool(translation_strings)
             rv.code_block = code_block
             rv.subparses = l.subparses
+            rv.init_priority = init_priority + l.init_offset
 
         finally:
             l.subparses = old_subparses
