@@ -1,4 +1,4 @@
-# Copyright 2004-2022 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2023 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -22,7 +22,7 @@
 from __future__ import division, absolute_import, with_statement, print_function, unicode_literals
 from renpy.compat import PY2, basestring, bchr, bord, chr, open, pystr, range, round, str, tobytes, unicode # *
 
-from typing import Optional, Callable
+from typing import Any, Optional, Callable
 
 import math
 
@@ -181,7 +181,8 @@ class TextSegment(object):
         if source is not None:
             self.antialias = source.antialias
             self.vertical = source.vertical
-            self.font = source.font
+            font = source.font
+            self.font = renpy.config.font_name_map.get(font, font)
             self.size = source.size
             self.bold = source.bold
             self.italic = source.italic
@@ -215,7 +216,8 @@ class TextSegment(object):
 
         self.antialias = style.antialias
         self.vertical = style.vertical
-        self.font = style.font
+        font = style.font
+        self.font = renpy.config.font_name_map.get(font, font)
         self.size = style.size
         self.bold = style.bold
         self.italic = style.italic
@@ -634,8 +636,15 @@ class Layout(object):
         started = self.start_segment is None
         ended = False
 
+        language = style.language
+
         for p_num, p in enumerate(self.paragraphs):
 
+            if language == "thaic90":
+                # Thai C90 - apply the Thai C90 algorithm to the text of each
+                # segment.
+                p = self.thaic90_paragraph(p)
+
             # RTL - apply RTL to the text of each segment, then
             # reverse the order of the segments in each paragraph.
             if renpy.config.rtl:
@@ -677,9 +686,8 @@ class Layout(object):
 
                 # Tag the glyphs that are eligible for line breaking, and if
                 # they should be included or excluded from the end of a line.
-                language = style.language
 
-                if language == "unicode" or language == "eastasian":
+                if language == "unicode" or language == "eastasian" or language == "thaic90":
                     textsupport.annotate_unicode(par_glyphs, False, 0)
                 elif language == "korean-with-spaces":
                     textsupport.annotate_unicode(par_glyphs, True, 0)
@@ -1175,6 +1183,7 @@ class Layout(object):
                     push().take_style(style, self)
 
                 elif tag == "font":
+                    value = renpy.config.font_name_map.get(value, value)
                     push().font = value
 
                 elif tag == "size":
@@ -1184,6 +1193,8 @@ class Layout(object):
 
                     if value[0] in "+-":
                         push().size += int(value)
+                    elif value[0] == "*":
+                        push().size = int(float(value[1:]) * push().size)
                     else:
                         push().size = int(value)
 
@@ -1288,6 +1299,23 @@ class Layout(object):
 
         return paragraphs
 
+    def thaic90_paragraph(self, p):
+        """
+        Given a paragraph (a list of (segment, text) tuples), converts the
+        text into c90-encoded thai text. This is an encoding that combines
+        multiple characters (base character, upper vowel, lower vowel, and
+        tone mark) into a single character in a unicode reserved space.
+        """
+
+        rv = [ ]
+
+        for ts, s in p:
+            s = renpy.text.extras.thaic90(s)
+            rv.append((ts, s))
+
+        return rv
+
+
     def rtl_paragraph(self, p):
         """
         Given a paragraph (a list of segment, text tuples) handles
@@ -1505,7 +1533,7 @@ class Text(renpy.display.core.Displayable):
     """
     :name: Text
     :doc: text
-    :args: (text, slow=None, scope=None, substitute=None, slow_done=None, **properties)
+    :args: (text, slow=None, scope=None, substitute=None, slow_done=None, *, tokenized=False, **properties)
 
     A displayable that displays text on the screen.
 
@@ -1530,6 +1558,10 @@ class Text(renpy.display.core.Displayable):
         If not None, and if slow text mode is enabled (see the `slow` parameter), this is a
         function or callable which is called with no arguments when the text finishes displaying.
 
+    `tokenized`
+        If true, `text` is expected to be a list of tokens, rather than a string. The tokens are
+        introduced in the :doc:`custom_text_tags` page.
+
     `**properties`
         Like other Displayables, Text takes style properties, including (among many others) the
         :propref:`mipmap` property.
@@ -1544,6 +1576,7 @@ class Text(renpy.display.core.Displayable):
     language = None
     mask = None
     last_ctc = None
+    tokenized = False
 
     def after_upgrade(self, version):
 
@@ -1561,25 +1594,28 @@ class Text(renpy.display.core.Displayable):
             self.end = None
             self.dirty = True
 
-    def __init__(self, text, slow=None, scope=None, substitute=None, slow_done=None, replaces=None, mask=None, **properties):
+    def __init__(self, text, slow=None, scope=None, substitute=None, slow_done=None, replaces=None, mask=None, tokenized=False, **properties):
 
         super(Text, self).__init__(**properties)
 
-        # We need text to be a list, so if it's not, wrap it.
-        if not isinstance(text, list):
-            text = [ text ]
 
-        # Check that the text is all text-able things.
-        for i in text:
-            if not isinstance(i, (basestring, renpy.display.core.Displayable)):
-                if renpy.config.developer:
-                    raise Exception("Cannot display {0!r} as text.".format(i))
-                else:
-                    text = [ "" ]
-                    break
+        if not tokenized:
+
+            # We need text to be a list, so if it's not, wrap it.
+            if not isinstance(text, list):
+                text = [ text ]
+
+            # Check that the text is all text-able things.
+            for i in text:
+                if not isinstance(i, (basestring, renpy.display.core.Displayable)):
+                    if renpy.config.developer:
+                        raise Exception("Cannot display {0!r} as text.".format(i))
+                    else:
+                        text = [ "" ]
+                        break
 
         # True if we are substituting things in.
-        self.substitute = substitute
+        self.substitute = substitute # type: bool | None
 
         # Do we need to update ourselves?
         self.dirty = True
@@ -1590,10 +1626,13 @@ class Text(renpy.display.core.Displayable):
         # A mask, for passwords and such.
         self.mask = mask
 
-        # Sets the text we're showing, and performs substitutions.
-        self.set_text(text, scope, substitute)
+        # True if the text is tokenized, False otherwise.
+        self.tokenized = tokenized
 
-        if renpy.game.less_updates or renpy.game.preferences.self_voicing:
+        # Sets the text we're showing, and performs substitutions.
+        self.set_text(text, scope, substitute) # type: ignore
+
+        if renpy.game.less_updates:
             slow = False
 
         # True if we're using slow text mode.
@@ -1687,13 +1726,25 @@ class Text(renpy.display.core.Displayable):
 
         return self.set_text(self.text_parameter, scope, self.substitute, update)
 
-    def set_text(self, text, scope=None, substitute=False, update=True):
+    def set_text(self, text, scope=None, substitute=False, update=True): # type: (Any, Any, bool|None, bool) -> bool
 
         if self.locked:
-            return
+            return False
 
         self.language = renpy.game.preferences.language
 
+        if self.tokenized:
+
+            if update and self.text != text:
+                self.dirty = True
+                renpy.display.render.redraw(self, 0)
+
+            self.text = text
+            self.text_parameter = text
+            self._uses_scope = False
+
+            return True
+
         old_text = self.text
 
         if not isinstance(text, list):
@@ -1709,7 +1760,7 @@ class Text(renpy.display.core.Displayable):
         for i in text:
             if isinstance(i, basestring):
                 if substitute is not False:
-                    i, did_sub = renpy.substitutions.substitute(i, scope, substitute)
+                    i, did_sub = renpy.substitutions.substitute(i, scope, substitute) # type: ignore
                     uses_scope = uses_scope or did_sub
 
                 if isinstance(i, bytes):
@@ -1759,60 +1810,65 @@ class Text(renpy.display.core.Displayable):
 
         self.kill_layout()
 
-        text = self.text
+        if not self.tokenized:
 
-        # Decide the portion of the text to show quickly, the part to
-        # show slowly, and the part not to show (but to lay out).
-        if self.start is not None:
-            start_string = text[0][:self.start]
-            mid_string = text[0][self.start:self.end]
-            end_string = text[0][self.end:]
+            text = self.text
 
-            if start_string:
-                start_string = start_string + "{_start}"
+            # Decide the portion of the text to show quickly, the part to
+            # show slowly, and the part not to show (but to lay out).
+            if self.start is not None:
+                start_string = text[0][:self.start]
+                mid_string = text[0][self.start:self.end]
+                end_string = text[0][self.end:]
 
-            if end_string:
-                end_string = "{_end}" + end_string
+                if start_string:
+                    start_string = start_string + "{_start}"
 
-            text_split = [ ]
+                if end_string:
+                    end_string = "{_end}" + end_string
 
-            if start_string:
-                text_split.append(start_string)
+                text_split = [ ]
 
-            text_split.append(mid_string)
+                if start_string:
+                    text_split.append(start_string)
 
-            if self.ctc is not None:
-                if isinstance(self.ctc, list):
-                    text_split.extend(self.ctc)
+                text_split.append(mid_string)
+
+                if self.ctc is not None:
+                    if isinstance(self.ctc, list):
+                        text_split.extend(self.ctc)
+                    else:
+                        text_split.append(self.ctc)
+
+                if end_string:
+                    text_split.append(end_string)
+
+                text_split.extend(text[1:])
+
+                text = text_split
+
+            else:
+                # Add the CTC.
+                if self.ctc is not None:
+                    if isinstance(self.ctc, list):
+                        text.extend(self.ctc)
+                    else:
+                        text.append(self.ctc)
+
+            if self.last_ctc is not None:
+                if isinstance(self.last_ctc, list):
+                    text.extend(self.last_ctc)
                 else:
-                    text_split.append(self.ctc)
+                    text.append(self.last_ctc)
 
-            if end_string:
-                text_split.append(end_string)
+            # Tokenize the text.
+            tokens = self.tokenize(text)
 
-            text_split.extend(text[1:])
-
-            text = text_split
+            if renpy.config.custom_text_tags or renpy.config.self_closing_custom_text_tags or (renpy.config.replace_text is not None):
+                tokens = self.apply_custom_tags(tokens)
 
         else:
-            # Add the CTC.
-            if self.ctc is not None:
-                if isinstance(self.ctc, list):
-                    text.extend(self.ctc)
-                else:
-                    text.append(self.ctc)
-
-        if self.last_ctc is not None:
-            if isinstance(self.last_ctc, list):
-                text.extend(self.last_ctc)
-            else:
-                text.append(self.last_ctc)
-
-        # Tokenize the text.
-        tokens = self.tokenize(text)
-
-        if renpy.config.custom_text_tags or renpy.config.self_closing_custom_text_tags or (renpy.config.replace_text is not None):
-            tokens = self.apply_custom_tags(tokens)
+            tokens = self.text
 
         # self.tokens is a list of pairs, where the first component of
         # each pair is TEXT, NEWLINE, TAG, or DISPLAYABLE, and the second
@@ -1936,7 +1992,7 @@ class Text(renpy.display.core.Displayable):
 
             layout = Layout(self, width, height, renders, size_only=True, drawable_res=True)
 
-        xpos, ypos, xanchor, yanchor, xoffset, yoffset, subpixel = rv
+        xpos, ypos, xanchor, _yanchor, xoffset, yoffset, subpixel = rv
         rv = (xpos, ypos, xanchor, layout.baseline, xoffset, yoffset, subpixel)
         return rv
 
@@ -1975,17 +2031,6 @@ class Text(renpy.display.core.Displayable):
         if hyperlink_focus and not default:
             return hyperlink_focus(None)
 
-    def call_slow_done(self, st):
-        """
-        Called when slow is finished.
-        """
-
-        self.slow = False
-
-        if self.slow_done:
-            self.slow_done()
-            self.slow_done = None
-
     def hyperlink_sensitive(self, target):
         """
         Returns true of the hyperlink is sensitive, False otherwise.
@@ -2003,11 +2048,28 @@ class Text(renpy.display.core.Displayable):
         Space, Enter, or Click ends slow, if it's enabled.
         """
 
+        if (self.slow_done is not None) and not self.slow:
+
+            if ev.type == renpy.display.core.TIMEEVENT and ev.modal:
+                renpy.game.interface.timeout(0.05)
+                return
+
+            try:
+                self.slow_done()
+            finally:
+                self.slow_done = None
+
         if self.slow and renpy.display.behavior.map_event(ev, "dismiss") and self.style.slow_abortable:
 
             for i in slow_text:
                 if i.slow:
-                    i.call_slow_done(st)
+                    i.slow = False
+
+                if i.slow_done is not None:
+                    try:
+                        i.slow_done()
+                    finally:
+                        i.slow_done = None
 
             raise renpy.display.core.IgnoreEvent()
 
@@ -2238,19 +2300,22 @@ class Text(renpy.display.core.Displayable):
             rv.blit(drend, (0, 0))
 
         # Add in the focus areas.
-        for hyperlink, hx, hy, hw, hh in layout.hyperlinks:
+        for hyperlink, hx, hy, hw, hh, valid_st in layout.hyperlinks:
 
-            h_x, h_y = layout.unscale_pair(hx + layout.xoffset, hy + layout.yoffset)
-            h_w, h_h = layout.unscale_pair(hw, hh)
+            if st >= valid_st or not self.slow:
 
-            rv.add_focus(self, hyperlink, h_x, h_y, h_w, h_h)
+                h_x, h_y = layout.unscale_pair(hx + layout.xoffset, hy + layout.yoffset)
+                h_w, h_h = layout.unscale_pair(hw, hh)
+
+                rv.add_focus(self, hyperlink, h_x, h_y, h_w, h_h)
 
         # Figure out if we need to redraw or call slow_done.
         if self.slow:
             if redraw is not None:
                 renpy.display.render.redraw(self, max(redraw, 0))
             else:
-                self.call_slow_done(st)
+                self.slow = False
+                renpy.game.interface.timeout(0)
 
         rv.forward = layout.forward
         rv.reverse = layout.reverse
@@ -2355,9 +2420,6 @@ class Text(renpy.display.core.Displayable):
 
                         contents.append(t2)
 
-                    if count:
-                        raise Exception("Text ended while the '{}' text tag was still open.".format(tag))
-
                     new_contents = func(tag, value, contents)
 
                 else:
